# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.identity import base
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators


class EndPointsTestJSON(base.BaseIdentityV3AdminTest):
    """Test keystone endpoints"""

    # NOTE: force_tenant_isolation is true in the base class by default but
    # overridden to false here to allow test execution for clouds using the
    # pre-provisioned credentials provider.
    force_tenant_isolation = False

    @classmethod
    def setup_clients(cls):
        super(EndPointsTestJSON, cls).setup_clients()
        cls.client = cls.endpoints_client

    @classmethod
    def resource_setup(cls):
        super(EndPointsTestJSON, cls).resource_setup()
        cls.service_ids = list()

        # Create endpoints so as to use for LIST and GET test cases
        interfaces = ['public', 'internal']
        cls.setup_endpoint_ids = list()
        for i in range(2):
            service = cls._create_service()
            cls.service_ids.append(service['id'])
            cls.addClassResourceCleanup(
                cls.services_client.delete_service, service['id'])

            region_name = data_utils.rand_name('region')
            url = data_utils.rand_url()
            endpoint = cls.client.create_endpoint(
                service_id=cls.service_ids[i], interface=interfaces[i],
                url=url, region=region_name, enabled=True)['endpoint']
            region = cls.regions_client.show_region(region_name)['region']
            cls.addClassResourceCleanup(
                cls.regions_client.delete_region, region['id'])
            cls.addClassResourceCleanup(
                cls.client.delete_endpoint, endpoint['id'])
            cls.setup_endpoint_ids.append(endpoint['id'])

    @classmethod
    def _create_service(cls, s_name=None, s_type=None, s_description=None):
        if s_name is None:
            s_name = data_utils.rand_name('service')
        if s_type is None:
            s_type = data_utils.rand_name('type')
        if s_description is None:
            s_description = data_utils.rand_name('description')
        service_data = (
            cls.services_client.create_service(name=s_name, type=s_type,
                                               description=s_description))
        return service_data['service']

    @decorators.idempotent_id('c19ecf90-240e-4e23-9966-21cee3f6a618')
    def test_list_endpoints(self):
        """Test listing keystone endpoints by filters"""
        # Get the list of all the endpoints.
        fetched_endpoints = self.client.list_endpoints()['endpoints']
        fetched_endpoint_ids = [e['id'] for e in fetched_endpoints]
        # Check that all the created endpoints are present in
        # "fetched_endpoints".
        missing_endpoints =\
            [e for e in self.setup_endpoint_ids
             if e not in fetched_endpoint_ids]
        self.assertEqual(0, len(missing_endpoints),
                         "Failed to find endpoint %s in fetched list" %
                         ', '.join(str(e) for e in missing_endpoints))

        # Check that filtering endpoints by service_id works.
        fetched_endpoints_for_service = self.client.list_endpoints(
            service_id=self.service_ids[0])['endpoints']
        fetched_endpoints_for_alt_service = self.client.list_endpoints(
            service_id=self.service_ids[1])['endpoints']

        # Assert that both filters returned the correct result.
        self.assertEqual(1, len(fetched_endpoints_for_service))
        self.assertEqual(1, len(fetched_endpoints_for_alt_service))
        self.assertEqual(set(self.setup_endpoint_ids),
                         set([fetched_endpoints_for_service[0]['id'],
                              fetched_endpoints_for_alt_service[0]['id']]))

        # Check that filtering endpoints by interface works.
        fetched_public_endpoints = self.client.list_endpoints(
            interface='public')['endpoints']
        fetched_internal_endpoints = self.client.list_endpoints(
            interface='internal')['endpoints']

        # Check that the expected endpoint_id is present per filter. [0] is
        # public and [1] is internal.
        self.assertIn(self.setup_endpoint_ids[0],
                      [e['id'] for e in fetched_public_endpoints])
        self.assertIn(self.setup_endpoint_ids[1],
                      [e['id'] for e in fetched_internal_endpoints])

    @decorators.idempotent_id('0e2446d2-c1fd-461b-a729-b9e73e3e3b37')
    def test_create_list_show_delete_endpoint(self):
        """Test creating, listing, showing and deleting keystone endpoint"""
        region_name = data_utils.rand_name('region')
        url = data_utils.rand_url()
        interface = 'public'
        endpoint = self.client.create_endpoint(service_id=self.service_ids[0],
                                               interface=interface,
                                               url=url, region=region_name,
                                               enabled=True)['endpoint']
        region = self.regions_client.show_region(region_name)['region']
        self.addCleanup(self.regions_client.delete_region, region['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.client.delete_endpoint, endpoint['id'])
        # Asserting Create Endpoint response body
        self.assertEqual(region_name, endpoint['region'])
        self.assertEqual(url, endpoint['url'])

        # Checking if created endpoint is present in the list of endpoints
        fetched_endpoints = self.client.list_endpoints()['endpoints']
        fetched_endpoints_id = [e['id'] for e in fetched_endpoints]
        self.assertIn(endpoint['id'], fetched_endpoints_id)

        # Show endpoint
        fetched_endpoint = (
            self.client.show_endpoint(endpoint['id'])['endpoint'])
        # Asserting if the attributes of endpoint are the same
        self.assertEqual(self.service_ids[0], fetched_endpoint['service_id'])
        self.assertEqual(interface, fetched_endpoint['interface'])
        self.assertEqual(url, fetched_endpoint['url'])
        self.assertEqual(region_name, fetched_endpoint['region'])
        self.assertEqual(True, fetched_endpoint['enabled'])

        # Deleting the endpoint created in this method
        self.client.delete_endpoint(endpoint['id'])

        # Checking whether endpoint is deleted successfully
        fetched_endpoints = self.client.list_endpoints()['endpoints']
        fetched_endpoints_id = [e['id'] for e in fetched_endpoints]
        self.assertNotIn(endpoint['id'], fetched_endpoints_id)

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('37e8f15e-ee7c-4657-a1e7-f6b61e375eff')
    def test_update_endpoint(self):
        """Test updating keystone endpoint"""
        # NOTE(zhufl) Service2 should be created before endpoint_for_update
        # is created, because Service2 must be deleted after
        # endpoint_for_update is deleted, otherwise we will get a 404 error
        # when deleting endpoint_for_update if endpoint's service is deleted.

        # Creating service for updating endpoint with new service ID
        s_name = data_utils.rand_name('service')
        s_type = data_utils.rand_name('type')
        s_description = data_utils.rand_name('description')
        service2 = self._create_service(s_name=s_name, s_type=s_type,
                                        s_description=s_description)
        self.addCleanup(self.services_client.delete_service, service2['id'])

        # Creating an endpoint so as to check update endpoint with new values
        region1_name = data_utils.rand_name('region')
        url1 = data_utils.rand_url()
        interface1 = 'public'
        endpoint_for_update = (
            self.client.create_endpoint(service_id=self.service_ids[0],
                                        interface=interface1,
                                        url=url1, region=region1_name,
                                        enabled=True)['endpoint'])
        region1 = self.regions_client.show_region(region1_name)['region']
        self.addCleanup(self.regions_client.delete_region, region1['id'])

        # Updating endpoint with new values
        region2_name = data_utils.rand_name('region')
        url2 = data_utils.rand_url()
        interface2 = 'internal'
        endpoint = self.client.update_endpoint(endpoint_for_update['id'],
                                               service_id=service2['id'],
                                               interface=interface2,
                                               url=url2, region=region2_name,
                                               enabled=False)['endpoint']
        region2 = self.regions_client.show_region(region2_name)['region']
        self.addCleanup(self.regions_client.delete_region, region2['id'])
        self.addCleanup(self.client.delete_endpoint, endpoint_for_update['id'])

        # Asserting if the attributes of endpoint are updated
        self.assertEqual(service2['id'], endpoint['service_id'])
        self.assertEqual(interface2, endpoint['interface'])
        self.assertEqual(url2, endpoint['url'])
        self.assertEqual(region2_name, endpoint['region'])
        self.assertEqual(False, endpoint['enabled'])



# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

_common_schema = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'certificate': {
                'type': 'object',
                'properties': {
                    'data': {'type': 'string'},
                    'private_key': {'type': 'string'},
                },
                'additionalProperties': False,
                'required': ['data', 'private_key']
            }
        },
        'additionalProperties': False,
        'required': ['certificate']
    }
}

get_certificate = copy.deepcopy(_common_schema)
get_certificate['response_body']['properties']['certificate'][
    'properties']['private_key'].update({'type': 'null'})

create_certificate = copy.deepcopy(_common_schema)



# Copyright 2013 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import re
import time
from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest import exceptions
from tempest.lib.common import rest_client
from tempest.lib import exceptions as lib_exc


class OrchestrationClient(rest_client.RestClient):

    def list_stacks(self, params=None):
        """Lists all stacks for a user."""

        uri = 'stacks'
        if params:
            uri += '?%s' % urllib.urlencode(params)

        resp, body = self.get(uri)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def create_stack(self, name, disable_rollback=True, parameters=None,
                     timeout_mins=60, template=None, template_url=None,
                     environment=None, files=None):
        if parameters is None:
            parameters = {}
        headers, body = self._prepare_update_create(
            name,
            disable_rollback,
            parameters,
            timeout_mins,
            template,
            template_url,
            environment,
            files)
        uri = 'stacks'
        resp, body = self.post(uri, headers=headers, body=body)
        self.expected_success(201, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def update_stack(self, stack_identifier, name, disable_rollback=True,
                     parameters=None, timeout_mins=60, template=None,
                     template_url=None, environment=None, files=None):
        if parameters is None:
            parameters = {}
        headers, body = self._prepare_update_create(
            name,
            disable_rollback,
            parameters,
            timeout_mins,
            template,
            template_url,
            environment)

        uri = "stacks/%s" % stack_identifier
        resp, body = self.put(uri, headers=headers, body=body)
        self.expected_success(202, resp.status)
        return rest_client.ResponseBody(resp, body)

    def _prepare_update_create(self, name, disable_rollback=True,
                               parameters=None, timeout_mins=60,
                               template=None, template_url=None,
                               environment=None, files=None):
        if parameters is None:
            parameters = {}
        post_body = {
            "stack_name": name,
            "disable_rollback": disable_rollback,
            "parameters": parameters,
            "timeout_mins": timeout_mins,
            "template": "HeatTemplateFormatVersion: '2012-12-12'\n",
            "environment": environment,
            "files": files
        }
        if template:
            post_body['template'] = template
        if template_url:
            post_body['template_url'] = template_url
        body = json.dumps(post_body)

        # Password must be provided on stack create so that heat
        # can perform future operations on behalf of the user
        headers = self.get_headers()
        headers['X-Auth-Key'] = self.password
        headers['X-Auth-User'] = self.user
        return headers, body

    def show_stack(self, stack_identifier):
        """Returns the details of a single stack."""
        url = "stacks/%s" % stack_identifier
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def suspend_stack(self, stack_identifier):
        """Suspend a stack."""
        url = 'stacks/%s/actions' % stack_identifier
        body = {'suspend': None}
        resp, body = self.post(url, json.dumps(body))
        self.expected_success(200, resp.status)
        return rest_client.ResponseBody(resp)

    def resume_stack(self, stack_identifier):
        """Resume a stack."""
        url = 'stacks/%s/actions' % stack_identifier
        body = {'resume': None}
        resp, body = self.post(url, json.dumps(body))
        self.expected_success(200, resp.status)
        return rest_client.ResponseBody(resp)

    def list_resources(self, stack_identifier):
        """Returns the details of a single resource."""
        url = "stacks/%s/resources" % stack_identifier
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_resource(self, stack_identifier, resource_name):
        """Returns the details of a single resource."""
        url = "stacks/%s/resources/%s" % (stack_identifier, resource_name)
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_stack(self, stack_identifier):
        """Deletes the specified Stack."""
        resp, _ = self.delete("stacks/%s" % str(stack_identifier))
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp)

    def wait_for_stack_status(self, stack_identifier, status,
                              failure_pattern='^.*_FAILED$'):
        """Waits for a Stack to reach a given status."""
        start = int(time.time())
        fail_regexp = re.compile(failure_pattern)

        while True:
            try:
                body = self.show_stack(stack_identifier)['stack']
            except lib_exc.NotFound:
                if status == 'DELETE_COMPLETE':
                    return
            stack_name = body['stack_name']
            stack_status = body['stack_status']
            if stack_status == status:
                return body
            if fail_regexp.search(stack_status):
                raise exceptions.StackBuildErrorException(
                    stack_identifier=stack_identifier,
                    stack_status=stack_status,
                    stack_status_reason=body['stack_status_reason'])

            if int(time.time()) - start >= self.build_timeout:
                message = ('Stack %s failed to reach %s status (current: %s) '
                           'within the required time (%s s).' %
                           (stack_name, status, stack_status,
                            self.build_timeout))
                raise lib_exc.TimeoutException(message)
            time.sleep(self.build_interval)

    def show_resource_metadata(self, stack_identifier, resource_name):
        """Returns the resource's metadata."""
        url = ('stacks/{stack_identifier}/resources/{resource_name}'
               '/metadata'.format(**locals()))
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_events(self, stack_identifier):
        """Returns list of all events for a stack."""
        url = 'stacks/{stack_identifier}/events'.format(**locals())
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_resource_events(self, stack_identifier, resource_name):
        """Returns list of all events for a resource from stack."""
        url = ('stacks/{stack_identifier}/resources/{resource_name}'
               '/events'.format(**locals()))
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_event(self, stack_identifier, resource_name, event_id):
        """Returns the details of a single stack's event."""
        url = ('stacks/{stack_identifier}/resources/{resource_name}/events'
               '/{event_id}'.format(**locals()))
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_template(self, stack_identifier):
        """Returns the template for the stack."""
        url = ('stacks/{stack_identifier}/template'.format(**locals()))
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def _validate_template(self, post_body):
        """Returns the validation request result."""
        post_body = json.dumps(post_body)
        resp, body = self.post('validate', post_body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def validate_template(self, template, parameters=None):
        """Returns the validation result for a template with parameters."""
        if parameters is None:
            parameters = {}
        post_body = {
            'template': template,
            'parameters': parameters,
        }
        return self._validate_template(post_body)

    def validate_template_url(self, template_url, parameters=None):
        """Returns the validation result for a template with parameters."""
        if parameters is None:
            parameters = {}
        post_body = {
            'template_url': template_url,
            'parameters': parameters,
        }
        return self._validate_template(post_body)

    def list_resource_types(self):
        """List resource types."""
        resp, body = self.get('resource_types')
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_resource_type(self, resource_type_name):
        """Return the schema of a resource type."""
        url = 'resource_types/%s' % resource_type_name
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        return rest_client.ResponseBody(resp, json.loads(body))

    def show_resource_type_template(self, resource_type_name):
        """Return the template of a resource type."""
        url = 'resource_types/%s/template' % resource_type_name
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        return rest_client.ResponseBody(resp, json.loads(body))

    def create_software_config(self, name=None, config=None, group=None,
                               inputs=None, outputs=None, options=None):
        headers, body = self._prep_software_config_create(
            name, config, group, inputs, outputs, options)

        url = 'software_configs'
        resp, body = self.post(url, headers=headers, body=body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_software_config(self, conf_id):
        """Returns a software configuration resource."""
        url = 'software_configs/%s' % str(conf_id)
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_software_config(self, conf_id):
        """Deletes a specific software configuration."""
        url = 'software_configs/%s' % str(conf_id)
        resp, _ = self.delete(url)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp)

    def create_software_deploy(self, server_id=None, config_id=None,
                               action=None, status=None,
                               input_values=None, output_values=None,
                               status_reason=None, signal_transport=None):
        """Creates or updates a software deployment."""
        headers, body = self._prep_software_deploy_update(
            None, server_id, config_id, action, status, input_values,
            output_values, status_reason, signal_transport)

        url = 'software_deployments'
        resp, body = self.post(url, headers=headers, body=body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def update_software_deploy(self, deploy_id=None, server_id=None,
                               config_id=None, action=None, status=None,
                               input_values=None, output_values=None,
                               status_reason=None, signal_transport=None):
        """Creates or updates a software deployment."""
        headers, body = self._prep_software_deploy_update(
            deploy_id, server_id, config_id, action, status, input_values,
            output_values, status_reason, signal_transport)

        url = 'software_deployments/%s' % str(deploy_id)
        resp, body = self.put(url, headers=headers, body=body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_software_deployments(self):
        """Returns a list of all deployments."""
        url = 'software_deployments'
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_software_deployment(self, deploy_id):
        """Returns a specific software deployment."""
        url = 'software_deployments/%s' % str(deploy_id)
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_software_deployment_metadata(self, server_id):
        """Return a config metadata for a specific server."""
        url = 'software_deployments/metadata/%s' % server_id
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_software_deploy(self, deploy_id):
        """Deletes a specific software deployment."""
        url = 'software_deployments/%s' % str(deploy_id)
        resp, _ = self.delete(url)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp)

    def _prep_software_config_create(self, name=None, conf=None, group=None,
                                     inputs=None, outputs=None, options=None):
        """Prepares a software configuration body."""
        post_body = {}
        if name is not None:
            post_body["name"] = name
        if conf is not None:
            post_body["config"] = conf
        if group is not None:
            post_body["group"] = group
        if inputs is not None:
            post_body["inputs"] = inputs
        if outputs is not None:
            post_body["outputs"] = outputs
        if options is not None:
            post_body["options"] = options
        body = json.dumps(post_body)

        headers = self.get_headers()
        return headers, body

    def _prep_software_deploy_update(self, deploy_id=None, server_id=None,
                                     config_id=None, action=None, status=None,
                                     input_values=None, output_values=None,
                                     status_reason=None,
                                     signal_transport=None):
        """Prepares a deployment create or update (if an id was given)."""
        post_body = {}

        if deploy_id is not None:
            post_body["id"] = deploy_id
        if server_id is not None:
            post_body["server_id"] = server_id
        if config_id is not None:
            post_body["config_id"] = config_id
        if action is not None:
            post_body["action"] = action
        if status is not None:
            post_body["status"] = status
        if input_values is not None:
            post_body["input_values"] = input_values
        if output_values is not None:
            post_body["output_values"] = output_values
        if status_reason is not None:
            post_body["status_reason"] = status_reason
        if signal_transport is not None:
            post_body["signal_transport"] = signal_transport
        body = json.dumps(post_body)

        headers = self.get_headers()
        return headers, body



# Copyright 2013 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from oslotest import base


class TestCase(base.BaseTestCase):

    def patch(self, target, *args, **kwargs):
        """Returns a started `mock.patch` object for the supplied target.

        The caller may then call the returned patcher to create a mock object.

        The caller does not need to call stop() on the returned
        patcher object, as this method automatically adds a cleanup
        to the test class to stop the patcher.

        :param target: string module.class or module.object expression to patch
        :param *args: passed as-is to `mock.patch`.
        :param **kwargs: passed as-is to `mock.patch`.

        See mock documentation for more details:
        https://docs.python.org/3.5/library/unittest.mock.html#unittest.mock.patch
        """

        p = mock.patch(target, *args, **kwargs)
        m = p.start()
        self.addCleanup(p.stop)
        return m

    def patchobject(self, target, attribute, *args, **kwargs):
        """Convenient wrapper around `mock.patch.object`

        Returns a started mock that will be automatically stopped after the
        test ran.

        :param target: object to have the attribute patched
        :param attribute: name of the attribute to be patched
        :param *args: passed as-is to `mock.patch.object`.
        :param **kwargs: passed as-is to `mock.patch.object`.

        See mock documentation for more details:
        https://docs.python.org/3.5/library/unittest.mock.html#unittest.mock.patch.object
        """

        p = mock.patch.object(target, attribute, *args, **kwargs)
        m = p.start()
        self.addCleanup(p.stop)
        return m



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class ImageMembersClient(rest_client.RestClient):
    api_version = "v1"

    def list_image_members(self, image_id):
        """List all members of an image."""
        url = 'images/%s/members' % image_id
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_shared_images(self, tenant_id):
        """List image memberships for the given tenant.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v1/#list-shared-images
        """

        url = 'shared-images/%s' % tenant_id
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def create_image_member(self, image_id, member_id, **kwargs):
        """Add a member to an image.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v1/#add-member-to-image
        """
        url = 'images/%s/members/%s' % (image_id, member_id)
        body = json.dumps({'member': kwargs})
        resp, __ = self.put(url, body)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp)

    def delete_image_member(self, image_id, member_id):
        """Removes a membership from the image.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v1/#remove-member
        """
        url = 'images/%s/members/%s' % (image_id, member_id)
        resp, __ = self.delete(url)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import re
import time

from oslo_log import log as logging

from tempest import config
from tempest.lib.common.utils.linux import remote_client
import tempest.lib.exceptions

CONF = config.CONF

LOG = logging.getLogger(__name__)


class RemoteClient(remote_client.RemoteClient):

    # TODO(oomichi): Make this class deprecated after migrating
    #                necessary methods to tempest.lib and cleaning
    #                unnecessary methods up from this class.
    def __init__(self, ip_address, username, password=None, pkey=None,
                 server=None, servers_client=None):
        """Executes commands in a VM over ssh

        :param ip_address: IP address to ssh to
        :param username: ssh username
        :param password: ssh password (optional)
        :param pkey: ssh public key (optional)
        :param server: server dict, used for debugging purposes
        :param servers_client: servers client, used for debugging purposes
        """
        super(RemoteClient, self).__init__(
            ip_address, username, password=password, pkey=pkey,
            server=server, servers_client=servers_client,
            ssh_timeout=CONF.validation.ssh_timeout,
            connect_timeout=CONF.validation.connect_timeout,
            console_output_enabled=CONF.compute_feature_enabled.console_output,
            ssh_shell_prologue=CONF.validation.ssh_shell_prologue,
            ping_count=CONF.validation.ping_count,
            ping_size=CONF.validation.ping_size)

    # Note that this method will not work on SLES11 guests, as they do
    # not support the TYPE column on lsblk
    def get_disks(self):
        # Select root disk devices as shown by lsblk
        command = 'lsblk -lb --nodeps'
        output = self.exec_command(command)
        selected = []
        pos = None
        for l in output.splitlines():
            if pos is None and l.find("TYPE") > 0:
                pos = l.find("TYPE")
                # Show header line too
                selected.append(l)
            # lsblk lists disk type in a column right-aligned with TYPE
            elif pos is not None and pos > 0 and l[pos:pos + 4] == "disk":
                selected.append(l)

        if selected:
            return "\n".join(selected)
        else:
            msg = "'TYPE' column is required but the output doesn't have it: "
            raise tempest.lib.exceptions.TempestException(msg + output)

    def list_disks(self):
        disks_list = self.get_disks()
        disks_list = [line[0] for line in
                      [device_name.split()
                       for device_name in disks_list.splitlines()][1:]]
        return disks_list

    def get_boot_time(self):
        cmd = 'cut -f1 -d. /proc/uptime'
        boot_secs = self.exec_command(cmd)
        boot_time = time.time() - int(boot_secs)
        return time.localtime(boot_time)

    def write_to_console(self, message):
        message = re.sub("([$\\`])", "\\\\\\\\\\1", message)
        # usually to /dev/ttyS0
        cmd = 'sudo sh -c "echo \\"%s\\" >/dev/console"' % message
        return self.exec_command(cmd)

    def get_mac_address(self, nic=""):
        show_nic = "show {nic} ".format(nic=nic) if nic else ""
        cmd = "ip addr %s| awk '/ether/ {print $2}'" % show_nic
        return self.exec_command(cmd).strip().lower()

    def get_nic_name_by_mac(self, address):
        cmd = "ip -o link | awk '/%s/ {print $2}'" % address
        nic = self.exec_command(cmd)
        return nic.strip().strip(":").split('@')[0].lower()

    def get_nic_name_by_ip(self, address):
        cmd = "ip -o addr | awk '/%s/ {print $2}'" % address
        nic = self.exec_command(cmd)
        LOG.debug('(get_nic_name_by_ip) Command result: %s', nic)
        return nic.strip().strip(":").split('@')[0].lower()

    def get_dns_servers(self):
        cmd = 'cat /etc/resolv.conf'
        resolve_file = self.exec_command(cmd).strip().split('\n')
        entries = (l.split() for l in resolve_file)
        dns_servers = [l[1] for l in entries
                       if len(l) and l[0] == 'nameserver']
        return dns_servers

    def _renew_lease_udhcpc(self, fixed_ip=None):
        """Renews DHCP lease via udhcpc client. """
        file_path = '/var/run/udhcpc.'
        nic_name = self.get_nic_name_by_ip(fixed_ip)
        pid = self.exec_command('cat {path}{nic}.pid'.
                                format(path=file_path, nic=nic_name))
        pid = pid.strip()
        cmd = 'sudo /bin/kill -{sig} {pid}'.format(pid=pid, sig='USR1')
        self.exec_command(cmd)

    def _renew_lease_dhclient(self, fixed_ip=None):
        """Renews DHCP lease via dhclient client. """
        cmd = "sudo /sbin/dhclient -r && sudo /sbin/dhclient"
        self.exec_command(cmd)

    def renew_lease(self, fixed_ip=None, dhcp_client='udhcpc'):
        """Wrapper method for renewing DHCP lease via given client

        Supporting:
        * udhcpc
        * dhclient
        """
        # TODO(yfried): add support for dhcpcd
        supported_clients = ['udhcpc', 'dhclient']
        if dhcp_client not in supported_clients:
            raise tempest.lib.exceptions.InvalidConfiguration(
                '%s DHCP client unsupported' % dhcp_client)
        if dhcp_client == 'udhcpc' and not fixed_ip:
            raise ValueError("need to set 'fixed_ip' for udhcpc client")
        return getattr(self, '_renew_lease_' + dhcp_client)(fixed_ip=fixed_ip)

    def mount(self, dev_name, mount_path='/mnt'):
        cmd_mount = 'sudo mount /dev/%s %s' % (dev_name, mount_path)
        self.exec_command(cmd_mount)

    def umount(self, mount_path='/mnt'):
        self.exec_command('sudo umount %s' % mount_path)

    def make_fs(self, dev_name, fs='ext4'):
        cmd_mkfs = 'sudo mke2fs -t %s /dev/%s' % (fs, dev_name)
        try:
            self.exec_command(cmd_mkfs)
        except tempest.lib.exceptions.SSHExecCommandFailed:
            LOG.error("Couldn't mke2fs")
            cmd_why = 'sudo ls -lR /dev'
            LOG.info("Contents of /dev: %s", self.exec_command(cmd_why))
            raise

    def nc_listen_host(self, port=80, protocol='tcp'):
        """Creates persistent nc server listening on the given TCP / UDP port

        :port: the port to start listening on.
        :protocol: the protocol used by the server. TCP by default.
        """
        udp = '-u' if protocol.lower() == 'udp' else ''
        cmd = "sudo nc %(udp)s -p %(port)s -lk -e echo foolish &" % {
            'udp': udp, 'port': port}
        return self.exec_command(cmd)

    def nc_host(self, host, port=80, protocol='tcp', expected_response=None):
        """Check connectivity to TCP / UDP port at host via nc

        :host: an IP against which the connectivity will be tested.
        :port: the port to check connectivity against.
        :protocol: the protocol used by nc to send packets. TCP by default.
        :expected_response: string representing the expected response
            from server.
        :raises SSHExecCommandFailed: if an expected response is given and it
            does not match the actual server response.
        """
        udp = '-u' if protocol.lower() == 'udp' else ''
        cmd = 'echo "bar" | nc -w 1 %(udp)s %(host)s %(port)s' % {
            'udp': udp, 'host': host, 'port': port}
        response = self.exec_command(cmd)

        # sending an UDP packet will always succeed. we need to check
        # the response.
        if (expected_response is not None and
                expected_response != response.strip()):
            raise tempest.lib.exceptions.SSHExecCommandFailed(
                command=cmd, exit_status=0, stdout=response, stderr='')
        return response

    def icmp_check(self, host, nic=None):
        """Wrapper for icmp connectivity checks"""
        return self.ping_host(host, nic=nic)

    def udp_check(self, host, **kwargs):
        """Wrapper for udp connectivity checks."""
        kwargs.pop('nic', None)
        return self.nc_host(host, protocol='udp', expected_response='foolish',
                            **kwargs)

    def tcp_check(self, host, **kwargs):
        """Wrapper for tcp connectivity checks."""
        kwargs.pop('nic', None)
        return self.nc_host(host, **kwargs)



# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from tempest.lib.services.identity.v3 import trusts_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestTrustsClient(base.BaseServiceTest):
    FAKE_CREATE_TRUST = {
        "trust": {
            "expires_at": "2013-02-27T18:30:59.999999Z",
            "impersonation": True,
            "allow_redelegation": True,
            "project_id": "ddef321",
            "roles": [
                {
                    "name": "member"
                    }
                ],
            "trustee_user_id": "86c0d5",
            "trustor_user_id": "a0fdfd"
            }
        }

    FAKE_LIST_TRUSTS = {
        "trusts": [
            {
                "id": "1ff900",
                "expires_at":
                "2013-02-27T18:30:59.999999Z",
                "impersonation": True,
                "links": {
                    "self":
                    "http://example.com/identity/v3/OS-TRUST/trusts/1ff900"
                    },
                "project_id": "0f1233",
                "trustee_user_id": "86c0d5",
                "trustor_user_id": "a0fdfd"
                },
            {
                "id": "f4513a",
                "impersonation": False,
                "links": {
                    "self":
                    "http://example.com/identity/v3/OS-TRUST/trusts/f45513a"
                    },
                "project_id": "0f1233",
                "trustee_user_id": "86c0d5",
                "trustor_user_id": "3cd2ce"
                }
            ]
        }

    FAKE_TRUST_INFO = {
        "trust": {
            "id": "987fe8",
            "expires_at": "2013-02-27T18:30:59.999999Z",
            "impersonation": True,
            "links": {
                "self":
                "http://example.com/identity/v3/OS-TRUST/trusts/987fe8"
                },
            "roles": [
                {
                    "id": "ed7b78",
                    "links": {
                        "self":
                        "http://example.com/identity/v3/roles/ed7b78"
                        },
                    "name": "member"
                    }
                ],
            "roles_links": {
                "next": None,
                "previous": None,
                "self":
                "http://example.com/identity/v3/OS-TRUST/trusts/1ff900/roles"
                },
            "project_id": "0f1233",
            "trustee_user_id": "be34d1",
            "trustor_user_id": "56ae32"
            }
        }

    FAKE_LIST_TRUSTS_ROLES = {
        "roles": [
            {
                "id": "c1648e",
                "links": {
                    "self": "http://example.com/identity/v3/roles/c1648e"
                    },
                "name": "manager"
                },
            {
                "id": "ed7b78",
                "links": {
                    "self": "http://example.com/identity/v3/roles/ed7b78"
                    },
                "name": "member"
                }
            ]
        }

    FAKE_TRUST_ROLE = {
        "role": {
            "id": "c1648e",
            "links": {
                "self": "http://example.com/identity/v3/roles/c1648e"
                },
            "name": "manager"
            }
        }

    def setUp(self):
        super(TestTrustsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = trusts_client.TrustsClient(fake_auth, 'identity',
                                                 'regionOne')

    def _test_create_trust(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_trust,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_CREATE_TRUST,
            bytes_body,
            status=201)

    def _test_show_trust(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_trust,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_TRUST_INFO,
            bytes_body,
            trust_id="1ff900")

    def _test_list_trusts(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_trusts,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_TRUSTS,
            bytes_body)

    def _test_list_trust_roles(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_trust_roles,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_TRUSTS_ROLES,
            bytes_body,
            trust_id="1ff900")

    def test_check_trust_role(self):
        self.check_service_client_function(
            self.client.check_trust_role,
            'tempest.lib.common.rest_client.RestClient.head',
            {},
            trust_id="1ff900",
            role_id="ed7b78")

    def _check_show_trust_role(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_trust_role,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_TRUST_ROLE,
            bytes_body,
            trust_id="1ff900",
            role_id="ed7b78")

    def test_list_trust_roles_with_str_body(self):
        self._test_list_trust_roles()

    def test_list_trust_roles_with_bytes_body(self):
        self._test_list_trust_roles(bytes_body=True)

    def test_check_show_trust_role_with_str_body(self):
        self._check_show_trust_role()

    def test_check_show_trust_role_with_bytes_body(self):
        self._check_show_trust_role(bytes_body=True)

    def test_create_trust_with_str_body(self):
        self._test_create_trust()

    def test_create_trust_with_bytes_body(self):
        self._test_create_trust(bytes_body=True)

    def test_show_trust_with_str_body(self):
        self._test_show_trust()

    def test_show_trust_with_bytes_body(self):
        self._test_show_trust(bytes_body=True)

    def test_list_trusts_with_str_body(self):
        self._test_list_trusts()

    def test_list_trusts_with_bytes_body(self):
        self._test_list_trusts(bytes_body=True)

    def test_delete_trust(self):
        self.check_service_client_function(
            self.client.delete_trust,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            trust_id="1ff900",
            status=204)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import io

from tempest.api.volume import base
from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class VolumesNegativeTest(base.BaseVolumeTest):
    """Negative tests of volumes"""

    @classmethod
    def resource_setup(cls):
        super(VolumesNegativeTest, cls).resource_setup()

        # Create a test shared instance and volume for attach/detach tests
        cls.volume = cls.create_volume()

    def create_image(self):
        # Create image
        image_name = data_utils.rand_name(self.__class__.__name__ + "-image")
        image = self.images_client.create_image(
            name=image_name,
            container_format=CONF.image.container_formats[0],
            disk_format=CONF.image.disk_formats[0],
            visibility='private',
            min_disk=CONF.volume.volume_size + 1)
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.images_client.delete_image, image['id'])

        # Upload image with 1KB data
        image_file = io.BytesIO(data_utils.random_bytes())
        self.images_client.store_image_file(image['id'], image_file)
        waiters.wait_for_image_status(self.images_client,
                                      image['id'], 'active')
        return image

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('f131c586-9448-44a4-a8b0-54ca838aa43e')
    def test_volume_get_nonexistent_volume_id(self):
        """Test getting non existent volume should fail"""
        self.assertRaises(lib_exc.NotFound, self.volumes_client.show_volume,
                          data_utils.rand_uuid())

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('555efa6e-efcd-44ef-8a3b-4a7ca4837a29')
    def test_volume_delete_nonexistent_volume_id(self):
        """Test deleting non existent volume should fail"""
        self.assertRaises(lib_exc.NotFound, self.volumes_client.delete_volume,
                          data_utils.rand_uuid())

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('1ed83a8a-682d-4dfb-a30e-ee63ffd6c049')
    def test_create_volume_with_invalid_size(self):
        """Test creating volume with invalid size should fail"""
        self.assertRaises(lib_exc.BadRequest,
                          self.volumes_client.create_volume, size='#$%')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('9387686f-334f-4d31-a439-33494b9e2683')
    def test_create_volume_without_passing_size(self):
        """Test creating volume with empty size should fail"""
        self.assertRaises(lib_exc.BadRequest,
                          self.volumes_client.create_volume, size='')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('41331caa-eaf4-4001-869d-bc18c1869360')
    def test_create_volume_with_size_zero(self):
        """Test creating volume with zero size should fail"""
        self.assertRaises(lib_exc.BadRequest,
                          self.volumes_client.create_volume, size='0')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('8b472729-9eba-446e-a83b-916bdb34bef7')
    def test_create_volume_with_size_negative(self):
        """Test creating volume with negative size should fail"""
        self.assertRaises(lib_exc.BadRequest,
                          self.volumes_client.create_volume, size='-1')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('10254ed8-3849-454e-862e-3ab8e6aa01d2')
    def test_create_volume_with_nonexistent_volume_type(self):
        """Test creating volume with non existent volume type should fail"""
        self.assertRaises(lib_exc.NotFound, self.volumes_client.create_volume,
                          size=CONF.volume.volume_size,
                          volume_type=data_utils.rand_uuid())

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('0c36f6ae-4604-4017-b0a9-34fdc63096f9')
    def test_create_volume_with_nonexistent_snapshot_id(self):
        """Test creating volume with non existent snapshot should fail"""
        self.assertRaises(lib_exc.NotFound, self.volumes_client.create_volume,
                          size=CONF.volume.volume_size,
                          snapshot_id=data_utils.rand_uuid())

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('47c73e08-4be8-45bb-bfdf-0c4e79b88344')
    def test_create_volume_with_nonexistent_source_volid(self):
        """Test creating volume with non existent source volume should fail"""
        self.assertRaises(lib_exc.NotFound, self.volumes_client.create_volume,
                          size=CONF.volume.volume_size,
                          source_volid=data_utils.rand_uuid())

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('0186422c-999a-480e-a026-6a665744c30c')
    def test_update_volume_with_nonexistent_volume_id(self):
        """Test updating non existent volume should fail"""
        self.assertRaises(lib_exc.NotFound, self.volumes_client.update_volume,
                          volume_id=data_utils.rand_uuid(), name="n")

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('e66e40d6-65e6-4e75-bdc7-636792fa152d')
    def test_update_volume_with_invalid_volume_id(self):
        """Test updating volume with invalid volume id should fail"""
        self.assertRaises(lib_exc.NotFound, self.volumes_client.update_volume,
                          volume_id=data_utils.rand_name('invalid'), name="n")

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('72aeca85-57a5-4c1f-9057-f320f9ea575b')
    def test_update_volume_with_empty_volume_id(self):
        """Test updating volume with empty volume id should fail"""
        self.assertRaises(lib_exc.NotFound, self.volumes_client.update_volume,
                          volume_id='')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('30799cfd-7ee4-446c-b66c-45b383ed211b')
    def test_get_invalid_volume_id(self):
        """Test getting volume with invalid volume id should fail"""
        self.assertRaises(lib_exc.NotFound, self.volumes_client.show_volume,
                          data_utils.rand_name('invalid'))

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('c6c3db06-29ad-4e91-beb0-2ab195fe49e3')
    def test_get_volume_without_passing_volume_id(self):
        """Test getting volume with empty volume id should fail"""
        self.assertRaises(lib_exc.NotFound,
                          self.volumes_client.show_volume, '')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('1f035827-7c32-4019-9240-b4ec2dbd9dfd')
    def test_delete_invalid_volume_id(self):
        """Test deleting volume with invalid volume id should fail"""
        self.assertRaises(lib_exc.NotFound, self.volumes_client.delete_volume,
                          data_utils.rand_name('invalid'))

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('441a1550-5d44-4b30-af0f-a6d402f52026')
    def test_delete_volume_without_passing_volume_id(self):
        """Test deleting volume with empty volume id should fail"""
        self.assertRaises(lib_exc.NotFound,
                          self.volumes_client.delete_volume, '')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('f5e56b0a-5d02-43c1-a2a7-c9b792c2e3f6')
    @utils.services('compute')
    def test_attach_volumes_with_nonexistent_volume_id(self):
        """Test attaching non existent volume to server should fail"""
        server = self.create_server()

        self.assertRaises(lib_exc.NotFound,
                          self.volumes_client.attach_volume,
                          data_utils.rand_uuid(),
                          instance_uuid=server['id'],
                          mountpoint="/dev/vdc")

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('9f9c24e4-011d-46b5-b992-952140ce237a')
    def test_detach_volumes_with_invalid_volume_id(self):
        """Test detaching volume with invalid volume id should fail"""
        self.assertRaises(lib_exc.NotFound,
                          self.volumes_client.detach_volume,
                          'xxx')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('e0c75c74-ee34-41a9-9288-2a2051452854')
    def test_volume_extend_with_size_smaller_than_original_size(self):
        """Test extending volume with decreasing size should fail"""
        extend_size = 0
        self.assertRaises(lib_exc.BadRequest,
                          self.volumes_client.extend_volume,
                          self.volume['id'], new_size=extend_size)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('5d0b480d-e833-439f-8a5a-96ad2ed6f22f')
    def test_volume_extend_with_non_number_size(self):
        """Test extending volume with non-integer size should fail"""
        extend_size = 'abc'
        self.assertRaises(lib_exc.BadRequest,
                          self.volumes_client.extend_volume,
                          self.volume['id'], new_size=extend_size)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('355218f1-8991-400a-a6bb-971239287d92')
    def test_volume_extend_with_None_size(self):
        """Test extending volume with none size should fail"""
        extend_size = None
        self.assertRaises(lib_exc.BadRequest,
                          self.volumes_client.extend_volume,
                          self.volume['id'], new_size=extend_size)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('8f05a943-013c-4063-ac71-7baf561e82eb')
    def test_volume_extend_with_nonexistent_volume_id(self):
        """Test extending non existent volume should fail"""
        extend_size = self.volume['size'] + 1
        self.assertRaises(lib_exc.NotFound, self.volumes_client.extend_volume,
                          data_utils.rand_uuid(), new_size=extend_size)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('aff8ba64-6d6f-4f2e-bc33-41a08ee9f115')
    def test_volume_extend_without_passing_volume_id(self):
        """Test extending volume without passing volume id should fail"""
        extend_size = self.volume['size'] + 1
        self.assertRaises(lib_exc.NotFound, self.volumes_client.extend_volume,
                          None, new_size=extend_size)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('ac6084c0-0546-45f9-b284-38a367e0e0e2')
    def test_reserve_volume_with_nonexistent_volume_id(self):
        """Test reserving non existent volume should fail"""
        self.assertRaises(lib_exc.NotFound,
                          self.volumes_client.reserve_volume,
                          data_utils.rand_uuid())

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('eb467654-3dc1-4a72-9b46-47c29d22654c')
    def test_unreserve_volume_with_nonexistent_volume_id(self):
        """Test unreserving non existent volume should fail"""
        self.assertRaises(lib_exc.NotFound,
                          self.volumes_client.unreserve_volume,
                          data_utils.rand_uuid())

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('449c4ed2-ecdd-47bb-98dc-072aeccf158c')
    def test_reserve_volume_with_negative_volume_status(self):
        """Test reserving already reserved volume should fail"""
        # Mark volume as reserved.
        self.volumes_client.reserve_volume(self.volume['id'])
        # Mark volume which is marked as reserved before
        self.assertRaises(lib_exc.BadRequest,
                          self.volumes_client.reserve_volume,
                          self.volume['id'])
        # Unmark volume as reserved.
        self.volumes_client.unreserve_volume(self.volume['id'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('0f4aa809-8c7b-418f-8fb3-84c7a5dfc52f')
    def test_list_volumes_with_nonexistent_name(self):
        """Test listing volumes with non existent name should get nothing"""
        v_name = data_utils.rand_name(self.__class__.__name__ + '-Volume')
        params = {'name': v_name}
        fetched_volume = self.volumes_client.list_volumes(
            params=params)['volumes']
        self.assertEmpty(fetched_volume)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('9ca17820-a0e7-4cbd-a7fa-f4468735e359')
    def test_list_volumes_detail_with_nonexistent_name(self):
        """Test listing volume details with non existent name

        Listing volume details with non existent name should get nothing.
        """
        v_name = data_utils.rand_name(self.__class__.__name__ + '-Volume')
        params = {'name': v_name}
        fetched_volume = \
            self.volumes_client.list_volumes(
                detail=True, params=params)['volumes']
        self.assertEmpty(fetched_volume)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('143b279b-7522-466b-81be-34a87d564a7c')
    def test_list_volumes_with_invalid_status(self):
        """Test listing volumes with invalid status should get nothing"""
        params = {'status': 'null'}
        fetched_volume = self.volumes_client.list_volumes(
            params=params)['volumes']
        self.assertEmpty(fetched_volume)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('ba94b27b-be3f-496c-a00e-0283b373fa75')
    def test_list_volumes_detail_with_invalid_status(self):
        """Test listing volume details with invalid status

        Listing volume details with invalid status should get nothing
        """
        params = {'status': 'null'}
        fetched_volume = \
            self.volumes_client.list_volumes(detail=True,
                                             params=params)['volumes']
        self.assertEmpty(fetched_volume)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('5b810c91-0ad1-47ce-aee8-615f789be78f')
    @utils.services('image')
    def test_create_volume_from_image_with_decreasing_size(self):
        """Test creating volume from image with decreasing size should fail"""
        # Create image
        image = self.create_image()

        # Note(jeremyZ): To shorten the test time (uploading a big size image
        # is time-consuming), here just consider the scenario that volume size
        # is smaller than the min_disk of image.
        self.assertRaises(lib_exc.BadRequest,
                          self.volumes_client.create_volume,
                          size=CONF.volume.volume_size,
                          imageRef=image['id'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('d15e7f35-2cfc-48c8-9418-c8223a89bcbb')
    @utils.services('image')
    def test_create_volume_from_deactivated_image(self):
        """Test creating volume from deactivated image should fail"""
        # Create image
        image = self.create_image()

        # Deactivate the image
        self.images_client.deactivate_image(image['id'])
        body = self.images_client.show_image(image['id'])
        self.assertEqual("deactivated", body['status'])
        # Try creating a volume from deactivated image
        self.assertRaises(lib_exc.BadRequest,
                          self.create_volume,
                          imageRef=image['id'])



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import time
from urllib import parse as urlparse

import testtools

from tempest.api.object_storage import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


# This test can be quite long to run due to its
# dependency on container-sync process running interval.
# You can obviously reduce the container-sync interval in the
# container-server configuration.


class ContainerSyncTest(base.BaseObjectTest):
    """Test container synchronization"""

    credentials = [['operator', CONF.object_storage.operator_role],
                   ['operator_alt', CONF.object_storage.operator_role]]

    @classmethod
    def setup_credentials(cls):
        super(ContainerSyncTest, cls).setup_credentials()
        cls.os_alt = cls.os_roles_operator_alt

    @classmethod
    def setup_clients(cls):
        super(ContainerSyncTest, cls).setup_clients()
        cls.object_client_alt = cls.os_alt.object_client
        cls.container_client_alt = cls.os_alt.container_client

    @classmethod
    def resource_setup(cls):
        super(ContainerSyncTest, cls).resource_setup()
        cls.containers = []
        cls.objects = []
        cls.clients = {}

        # Default container-server config only allows localhost
        cls.local_ip = '127.0.0.1'

        # Must be configure according to container-sync interval
        container_sync_timeout = CONF.object_storage.container_sync_timeout
        cls.container_sync_interval = \
            CONF.object_storage.container_sync_interval
        cls.attempts = \
            int(container_sync_timeout / cls.container_sync_interval)

        # define container and object clients
        cls.clients[data_utils.rand_name(name='TestContainerSync')] = \
            (cls.container_client, cls.object_client)
        cls.clients[data_utils.rand_name(name='TestContainerSync')] = \
            (cls.container_client_alt, cls.object_client_alt)
        for cont_name, client in cls.clients.items():
            client[0].create_container(cont_name)
            cls.addClassResourceCleanup(base.delete_containers,
                                        cont_name,
                                        client[0],
                                        client[1])
            cls.containers.append(cont_name)

    def _test_container_synchronization(self, make_headers):
        # container to container synchronization
        # to allow/accept sync requests to/from other accounts

        # turn container synchronization on and create object in container
        for cont in (self.containers, self.containers[::-1]):
            cont_client = [self.clients[c][0] for c in cont]
            obj_client = [self.clients[c][1] for c in cont]
            headers = make_headers(cont[1], cont_client[1])
            cont_client[0].put(str(cont[0]), body=None, headers=headers)
            # create object in container
            object_name = data_utils.rand_name(name='TestSyncObject')
            data = object_name[::-1].encode()  # Raw data, we need bytes
            obj_client[0].create_object(cont[0], object_name, data)
            self.objects.append(object_name)

        # wait until container contents list is not empty
        cont_client = [self.clients[c][0] for c in self.containers]
        params = {'format': 'json'}
        while self.attempts > 0:
            object_lists = []
            for c_client, cont in zip(cont_client, self.containers):
                resp, object_list = c_client.list_container_objects(
                    cont, params=params)
                object_lists.append(dict(
                    (obj['name'], obj) for obj in object_list))
            # check that containers are not empty and have equal keys()
            # or wait for next attempt
            if object_lists[0] and object_lists[1] and \
                    set(object_lists[0].keys()) == set(object_lists[1].keys()):
                break
            else:
                time.sleep(self.container_sync_interval)
                self.attempts -= 1

        self.assertEqual(object_lists[0], object_lists[1],
                         'Different object lists in containers.')

        # Verify object content
        obj_clients = [(self.clients[c][1], c) for c in self.containers]
        for obj_client, cont in obj_clients:
            for obj_name in object_lists[0]:
                resp, object_content = obj_client.get_object(cont, obj_name)
                self.assertEqual(object_content, obj_name[::-1].encode())

    @decorators.attr(type='slow')
    @decorators.unstable_test(bug='1317133')
    @decorators.idempotent_id('be008325-1bba-4925-b7dd-93b58f22ce9b')
    @testtools.skipIf(
        not CONF.object_storage_feature_enabled.container_sync,
        'Old-style container sync function is disabled')
    def test_container_synchronization(self):
        """Test container synchronization"""
        def make_headers(cont, cont_client):
            # tell first container to synchronize to a second
            client_proxy_ip = \
                urlparse.urlparse(cont_client.base_url).netloc.split(':')[0]
            client_base_url = \
                cont_client.base_url.replace(client_proxy_ip,
                                             self.local_ip)
            headers = {'X-Container-Sync-Key': 'sync_key',
                       'X-Container-Sync-To': "%s/%s" %
                       (client_base_url, str(cont))}
            return headers
        self._test_container_synchronization(make_headers)



# Copyright 2017 IBM Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from tempest.tests import base


class ConfCounter(object):

    def __init__(self, *args, **kwargs):
        self.count = 0

    def __getattr__(self, key):
        self.count += 1
        return mock.MagicMock()

    def get_counts(self):
        return self.count


class TestImports(base.TestCase):
    def setUp(self):
        super(TestImports, self).setUp()
        self.conf_mock = self.patch('tempest.config.CONF',
                                    new_callable=ConfCounter)

    def test_account_generator_command_import(self):
        from tempest.cmd import account_generator  # noqa
        self.assertEqual(0, self.conf_mock.get_counts())

    def test_cleanup_command_import(self):
        from tempest.cmd import cleanup  # noqa
        self.assertEqual(0, self.conf_mock.get_counts())

    def test_init_command_import(self):
        from tempest.cmd import init  # noqa
        self.assertEqual(0, self.conf_mock.get_counts())

    def test_list_plugins_command_import(self):
        from tempest.cmd import list_plugins  # noqa
        self.assertEqual(0, self.conf_mock.get_counts())

    def test_run_command_import(self):
        from tempest.cmd import run  # noqa
        self.assertEqual(0, self.conf_mock.get_counts())

    def test_subunit_descibe_command_import(self):
        from tempest.cmd import subunit_describe_calls  # noqa
        self.assertEqual(0, self.conf_mock.get_counts())

    def test_verify_tempest_config_command_import(self):
        from tempest.cmd import verify_tempest_config  # noqa
        self.assertEqual(0, self.conf_mock.get_counts())

    def test_workspace_command_import(self):
        from tempest.cmd import workspace  # noqa
        self.assertEqual(0, self.conf_mock.get_counts())



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.network import base
from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class RoutersNegativeTest(base.BaseNetworkTest):
    """Negative tests of routers"""

    @classmethod
    def skip_checks(cls):
        super(RoutersNegativeTest, cls).skip_checks()
        if not utils.is_extension_enabled('router', 'network'):
            msg = "router extension not enabled."
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(RoutersNegativeTest, cls).resource_setup()
        cls.router = cls.create_router()
        cls.network = cls.create_network()
        cls.subnet = cls.create_subnet(cls.network)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('37a94fc0-a834-45b9-bd23-9a81d2fd1e22')
    def test_router_add_gateway_invalid_network_returns_404(self):
        """Test adding gateway with invalid network for router"""
        self.assertRaises(lib_exc.NotFound,
                          self.routers_client.update_router,
                          self.router['id'],
                          external_gateway_info={
                              'network_id': self.router['id']})

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('11836a18-0b15-4327-a50b-f0d9dc66bddd')
    def test_router_add_gateway_net_not_external_returns_400(self):
        """Test adding gateway with not external network for router"""
        alt_network = self.create_network()
        sub_cidr = self.cidr.next()
        self.create_subnet(alt_network, cidr=sub_cidr)
        self.assertRaises(lib_exc.BadRequest,
                          self.routers_client.update_router,
                          self.router['id'],
                          external_gateway_info={
                              'network_id': alt_network['id']})

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('957751a3-3c68-4fa2-93b6-eb52ea10db6e')
    def test_add_router_interfaces_on_overlapping_subnets_returns_400(self):
        """Test adding router interface which is on overlapping subnets"""
        network01 = self.create_network(
            network_name=data_utils.rand_name('router-network01-'))
        network02 = self.create_network(
            network_name=data_utils.rand_name('router-network02-'))
        subnet01 = self.create_subnet(network01)
        subnet02 = self.create_subnet(network02)
        interface = self.routers_client.add_router_interface(
            self.router['id'], subnet_id=subnet01['id'])
        self.addCleanup(self.routers_client.remove_router_interface,
                        self.router['id'], subnet_id=subnet01['id'])
        self.assertEqual(subnet01['id'], interface['subnet_id'])
        self.assertRaises(lib_exc.BadRequest,
                          self.routers_client.add_router_interface,
                          self.router['id'],
                          subnet_id=subnet02['id'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('04df80f9-224d-47f5-837a-bf23e33d1c20')
    def test_router_remove_interface_in_use_returns_409(self):
        """Test removing in-use interface from router"""
        self.routers_client.add_router_interface(self.router['id'],
                                                 subnet_id=self.subnet['id'])
        self.addCleanup(self.routers_client.remove_router_interface,
                        self.router['id'], subnet_id=self.subnet['id'])
        self.assertRaises(lib_exc.Conflict,
                          self.routers_client.delete_router,
                          self.router['id'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('c2a70d72-8826-43a7-8208-0209e6360c47')
    def test_show_non_existent_router_returns_404(self):
        """Test showing non existent router"""
        router = data_utils.rand_name('non_exist_router')
        self.assertRaises(lib_exc.NotFound, self.routers_client.show_router,
                          router)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('b23d1569-8b0c-4169-8d4b-6abd34fad5c7')
    def test_update_non_existent_router_returns_404(self):
        """Test updating non existent router"""
        router = data_utils.rand_name('non_exist_router')
        self.assertRaises(lib_exc.NotFound, self.routers_client.update_router,
                          router, name="new_name")

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('c7edc5ad-d09d-41e6-a344-5c0c31e2e3e4')
    def test_delete_non_existent_router_returns_404(self):
        """Test deleting non existent router"""
        router = data_utils.rand_name('non_exist_router')
        self.assertRaises(lib_exc.NotFound, self.routers_client.delete_router,
                          router)


class RoutersNegativeIpV6Test(RoutersNegativeTest):
    _ip_version = 6


class DvrRoutersNegativeTest(base.BaseNetworkTest):
    """Negative tests of DVR router"""

    @classmethod
    def skip_checks(cls):
        super(DvrRoutersNegativeTest, cls).skip_checks()
        if not utils.is_extension_enabled('dvr', 'network'):
            msg = "DVR extension not enabled."
            raise cls.skipException(msg)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('4990b055-8fc7-48ab-bba7-aa28beaad0b9')
    def test_router_create_tenant_distributed_returns_forbidden(self):
        """Non admin user is not allowed to create distributed router"""
        self.assertRaises(lib_exc.Forbidden, self.create_router,
                          distributed=True)



# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class EndpointsClient(rest_client.RestClient):
    api_version = "v2.0"

    def create_endpoint(self, **kwargs):
        """Create an endpoint for service.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v2-admin/index.html#create-endpoint-template
        """

        post_body = json.dumps({'endpoint': kwargs})
        resp, body = self.post('/endpoints', post_body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_endpoints(self):
        """List Endpoints - Returns Endpoints."""
        resp, body = self.get('/endpoints')
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_endpoint(self, endpoint_id):
        """Delete an endpoint."""
        url = '/endpoints/%s' % endpoint_id
        resp, body = self.delete(url)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)



# Copyright 2014 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest import config
from tempest.lib import decorators

CONF = config.CONF


class NetworksTest(base.BaseV2ComputeAdminTest):
    """Tests Nova Networks API that usually requires admin privileges.

    API docs:
    https://docs.openstack.org/api-ref/compute/#networks-os-networks-deprecated
    """
    max_microversion = '2.35'

    @classmethod
    def setup_clients(cls):
        super(NetworksTest, cls).setup_clients()
        cls.client = cls.os_admin.compute_networks_client

    @decorators.idempotent_id('d206d211-8912-486f-86e2-a9d090d1f416')
    def test_get_network(self):
        """Test getting network from nova side"""
        networks = self.client.list_networks()['networks']
        if CONF.compute.fixed_network_name:
            configured_network = [x for x in networks if x['label'] ==
                                  CONF.compute.fixed_network_name]
            self.assertEqual(1, len(configured_network),
                             "{0} networks with label {1}".format(
                                 len(configured_network),
                                 CONF.compute.fixed_network_name))
        elif CONF.network.public_network_id:
            configured_network = [x for x in networks if x['id'] ==
                                  CONF.network.public_network_id]
        else:
            raise self.skipException(
                "Environment has no known-for-sure existing network.")
        configured_network = configured_network[0]
        network = (self.client.show_network(configured_network['id'])
                   ['network'])
        self.assertEqual(configured_network['label'], network['label'])

    @decorators.idempotent_id('df3d1046-6fa5-4b2c-ad0c-cfa46a351cb9')
    def test_list_all_networks(self):
        """Test getting all networks from nova side"""
        networks = self.client.list_networks()['networks']
        # Check the configured network is in the list
        if CONF.compute.fixed_network_name:
            configured_network = CONF.compute.fixed_network_name
            self.assertIn(configured_network, [x['label'] for x in networks])
        else:
            network_labels = [x['label'] for x in networks]
            self.assertNotEmpty(network_labels)



# Copyright 2016 EasyStack. All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.image.v2 import namespace_tags_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestNamespaceTagsClient(base.BaseServiceTest):
    FAKE_CREATE_SHOW_TAGS = {
        "created_at": "2015-05-09T01:12:31Z",
        "name": "added-sample-tag",
        "updated_at": "2015-05-09T01:12:31Z"
    }

    FAKE_LIST_TAGS = {
        "tags": [
            {
                "name": "sample-tag1"
            },
            {
                "name": "sample-tag2"
            },
            {
                "name": "sample-tag3"
            }
        ]
    }

    FAKE_UPDATE_TAGS = {"name": "new-tag-name"}

    def setUp(self):
        super(TestNamespaceTagsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = namespace_tags_client.NamespaceTagsClient(
            fake_auth, 'image', 'regionOne')

    def _test_create_namespace_tags(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_namespace_tags,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_CREATE_SHOW_TAGS,
            bytes_body, status=201,
            namespace="OS::Compute::Hypervisor",
            tags=[{"name": "sample-tag1"},
                  {"name": "sample-tag2"},
                  {"name": "sample-tag3"}])

    def _test_list_namespace_tags(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_namespace_tags,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_TAGS,
            bytes_body,
            namespace="OS::Compute::Hypervisor")

    def _test_create_namespace_tag_definition(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_namespace_tag,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_CREATE_SHOW_TAGS,
            bytes_body,
            status=201,
            namespace="OS::Compute::Hypervisor",
            tag_name="added-sample-tag")

    def _test_show_namespace_tag_definition(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_namespace_tag,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_CREATE_SHOW_TAGS,
            bytes_body,
            namespace="OS::Compute::Hypervisor",
            tag_name="added-sample-tag")

    def _test_update_namespace_tag_definition(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_namespace_tag,
            'tempest.lib.common.rest_client.RestClient.put',
            self.FAKE_UPDATE_OBJECTS,
            bytes_body,
            namespace="OS::Compute::Hypervisor",
            tag_name="added-sample-tag",
            name="new-tag-name")

    def test_create_namespace_tags_with_str_body(self):
        self._test_create_namespace_tags()

    def test_create_namespace_tags_with_bytes_body(self):
        self._test_create_namespace_tags(bytes_body=True)

    def test_list_namespace_tags_with_str_body(self):
        self._test_list_namespace_tags()

    def test_list_namespace_tags_with_bytes_body(self):
        self._test_list_namespace_tags(bytes_body=True)

    def test_create_namespace_tag_with_str_body(self):
        self._test_create_namespace_tag_definition()

    def test_create_namespace_tag_with_bytes_body(self):
        self._test_create_namespace_tag_definition(bytes_body=True)

    def test_show_namespace_tag_with_str_body(self):
        self._test_show_namespace_tag_definition()

    def test_show_namespace_tag_with_bytes_body(self):
        self._test_show_namespace_tag_definition(bytes_body=True)

    def test_delete_namespace_tag_definition(self):
        self.check_service_client_function(
            self.client.delete_namespace_tag,
            'tempest.lib.common.rest_client.RestClient.delete',
            {}, status=204,
            namespace="OS::Compute::Hypervisor",
            tag_name="added-sample-tag")

    def test_delete_all_namespace_tags(self):
        self.check_service_client_function(
            self.client.delete_namespace_tags,
            'tempest.lib.common.rest_client.RestClient.delete',
            {}, status=204,
            namespace="OS::Compute::Hypervisor")



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.volume import base
from tempest.common import waiters
from tempest.lib.common.utils import data_utils as utils
from tempest.lib import decorators


class QosSpecsTestJSON(base.BaseVolumeAdminTest):
    """Test the Cinder QoS-specs.

    Tests for  create, list, delete, show, associate,
    disassociate, set/unset key APIs.
    """

    @classmethod
    def resource_setup(cls):
        super(QosSpecsTestJSON, cls).resource_setup()
        # Create admin qos client
        # Create a test shared qos-specs for tests
        cls.qos_name = utils.rand_name(cls.__name__ + '-QoS')
        cls.qos_consumer = 'front-end'

        cls.created_qos = cls.create_test_qos_specs(cls.qos_name,
                                                    cls.qos_consumer,
                                                    read_iops_sec='2000')

    def _create_delete_test_qos_with_given_consumer(self, consumer):
        name = utils.rand_name(self.__class__.__name__ + '-qos')
        qos = {'name': name, 'consumer': consumer}
        body = self.create_test_qos_specs(name, consumer)
        for key in ['name', 'consumer']:
            self.assertEqual(qos[key], body[key])

        self.admin_volume_qos_client.delete_qos(body['id'])
        self.admin_volume_qos_client.wait_for_resource_deletion(body['id'])

        # validate the deletion
        list_qos = self.admin_volume_qos_client.list_qos()['qos_specs']
        self.assertNotIn(body, list_qos)

    def _test_associate_qos(self, vol_type_id):
        self.admin_volume_qos_client.associate_qos(
            self.created_qos['id'], vol_type_id)

    @decorators.idempotent_id('7e15f883-4bef-49a9-95eb-f94209a1ced1')
    def test_create_delete_qos_with_front_end_consumer(self):
        """Tests the creation and deletion of QoS specs

        With consumer as front end
        """
        self._create_delete_test_qos_with_given_consumer('front-end')

    @decorators.idempotent_id('b115cded-8f58-4ee4-aab5-9192cfada08f')
    def test_create_delete_qos_with_back_end_consumer(self):
        """Tests the creation and deletion of QoS specs

        With consumer as back-end
        """
        self._create_delete_test_qos_with_given_consumer('back-end')

    @decorators.idempotent_id('f88d65eb-ea0d-487d-af8d-71f4011575a4')
    def test_create_delete_qos_with_both_consumer(self):
        """Tests the creation and deletion of QoS specs

        With consumer as both front end and back end
        """
        self._create_delete_test_qos_with_given_consumer('both')

    @decorators.idempotent_id('7aa214cc-ac1a-4397-931f-3bb2e83bb0fd')
    def test_get_qos(self):
        """Tests the detail of a given qos-specs"""
        body = self.admin_volume_qos_client.show_qos(
            self.created_qos['id'])['qos_specs']
        self.assertEqual(self.qos_name, body['name'])
        self.assertEqual(self.qos_consumer, body['consumer'])

    @decorators.idempotent_id('75e04226-bcf7-4595-a34b-fdf0736f38fc')
    def test_list_qos(self):
        """Tests the list of all qos-specs"""
        body = self.admin_volume_qos_client.list_qos()['qos_specs']
        self.assertIn(self.created_qos, body)

    @decorators.idempotent_id('ed00fd85-4494-45f2-8ceb-9e2048919aed')
    def test_set_unset_qos_key(self):
        """Test the addition of a specs key to qos-specs"""
        args = {'iops_bytes': '500'}
        body = self.admin_volume_qos_client.set_qos_key(
            self.created_qos['id'],
            iops_bytes='500')['qos_specs']
        self.assertEqual(args, body)
        body = self.admin_volume_qos_client.show_qos(
            self.created_qos['id'])['qos_specs']
        self.assertEqual(args['iops_bytes'], body['specs']['iops_bytes'])

        # test the deletion of a specs key from qos-specs
        keys = ['iops_bytes']
        self.admin_volume_qos_client.unset_qos_key(self.created_qos['id'],
                                                   keys)
        operation = 'qos-key-unset'
        waiters.wait_for_qos_operations(self.admin_volume_qos_client,
                                        self.created_qos['id'],
                                        operation, keys)
        body = self.admin_volume_qos_client.show_qos(
            self.created_qos['id'])['qos_specs']
        self.assertNotIn(keys[0], body['specs'])

    @decorators.idempotent_id('1dd93c76-6420-485d-a771-874044c416ac')
    def test_associate_disassociate_qos(self):
        """Test the following operations :

        1. associate_qos
        2. get_association_qos
        3. disassociate_qos
        4. disassociate_all_qos
        """

        # create a test volume-type
        vol_type = []
        for _ in range(0, 3):
            vol_type.append(self.create_volume_type())

        # associate the qos-specs with volume-types
        for i in range(0, 3):
            self._test_associate_qos(vol_type[i]['id'])

        # get the association of the qos-specs
        body = self.admin_volume_qos_client.show_association_qos(
            self.created_qos['id'])['qos_associations']
        associations = [association['id'] for association in body]
        for i in range(0, 3):
            self.assertIn(vol_type[i]['id'], associations)

        # disassociate a volume-type with qos-specs
        self.admin_volume_qos_client.disassociate_qos(
            self.created_qos['id'], vol_type[0]['id'])
        operation = 'disassociate'
        waiters.wait_for_qos_operations(self.admin_volume_qos_client,
                                        self.created_qos['id'], operation,
                                        vol_type[0]['id'])

        # disassociate all volume-types from qos-specs
        self.admin_volume_qos_client.disassociate_all_qos(
            self.created_qos['id'])
        operation = 'disassociate-all'
        waiters.wait_for_qos_operations(self.admin_volume_qos_client,
                                        self.created_qos['id'], operation)



# Copyright 2014 Cisco Systems, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log as logging

from tempest.common import utils
from tempest import config
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions
from tempest.scenario import manager

CONF = config.CONF
LOG = logging.getLogger(__name__)


class TestGettingAddress(manager.NetworkScenarioTest):
    """Test Summary:

    1. Create network with subnets:
        1.1. one IPv4 and
        1.2. one or more IPv6 in a given address mode
    2. Boot 2 VMs on this network
    3. Allocate and assign 2 FIP4
    4. Check that vNICs of all VMs gets all addresses actually assigned
    5. Each VM will ping the other's v4 private address
    6. If ping6 available in VM, each VM will ping all of the other's  v6
       addresses as well as the router's
    """

    @classmethod
    def skip_checks(cls):
        super(TestGettingAddress, cls).skip_checks()
        if not (CONF.network_feature_enabled.ipv6 and
                CONF.network_feature_enabled.ipv6_subnet_attributes):
            raise cls.skipException('IPv6 or its attributes not supported')
        if not (CONF.network.project_networks_reachable or
                CONF.network.public_network_id):
            msg = ('Either project_networks_reachable must be "true", or '
                   'public_network_id must be defined.')
            raise cls.skipException(msg)
        if CONF.network.shared_physical_network:
            msg = 'Deployment uses a shared physical network'
            raise cls.skipException(msg)
        if not CONF.network_feature_enabled.floating_ips:
            raise cls.skipException("Floating ips are not available")

    @classmethod
    def setup_credentials(cls):
        # Create no network resources for these tests.
        cls.set_network_resources()
        super(TestGettingAddress, cls).setup_credentials()

    def setUp(self):
        super(TestGettingAddress, self).setUp()
        self.keypair = self.create_keypair()
        self.sec_grp = self.create_security_group()

    def prepare_network(self, address6_mode, n_subnets6=1, dualnet=False):
        """Prepare network

        Creates network with given number of IPv6 subnets in the given mode and
        one IPv4 subnet.
        Creates router with ports on all subnets.
        if dualnet - create IPv6 subnets on a different network
        :return: list of created networks
        """
        network = self.create_network()
        if dualnet:
            network_v6 = self.create_network()

        sub4 = self.create_subnet(network=network,
                                  namestart='sub4',
                                  ip_version=4)

        router = self.get_router()
        self.routers_client.add_router_interface(router['id'],
                                                 subnet_id=sub4['id'])

        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.routers_client.remove_router_interface,
                        router['id'], subnet_id=sub4['id'])

        self.subnets_v6 = []
        for _ in range(n_subnets6):
            net6 = network_v6 if dualnet else network
            sub6 = self.create_subnet(network=net6,
                                      namestart='sub6',
                                      ip_version=6,
                                      ipv6_ra_mode=address6_mode,
                                      ipv6_address_mode=address6_mode)

            self.routers_client.add_router_interface(router['id'],
                                                     subnet_id=sub6['id'])

            self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                            self.routers_client.remove_router_interface,
                            router['id'], subnet_id=sub6['id'])

            self.subnets_v6.append(sub6)
        return [network, network_v6] if dualnet else [network]

    @staticmethod
    def define_server_ips(srv):
        ips = {'4': None, '6': []}
        for nics in srv['addresses'].values():
            for nic in nics:
                if nic['version'] == 6:
                    ips['6'].append(nic['addr'])
                else:
                    ips['4'] = nic['addr']
        return ips

    def prepare_server(self, networks=None):
        username = CONF.validation.image_ssh_user

        srv = self.create_server(
            key_name=self.keypair['name'],
            security_groups=[{'name': self.sec_grp['name']}],
            networks=[{'uuid': n['id']} for n in networks])
        fip = self.create_floating_ip(server=srv)
        ips = self.define_server_ips(srv=srv)
        ssh = self.get_remote_client(
            ip_address=fip['floating_ip_address'],
            username=username, server=srv)
        return ssh, ips, srv

    def turn_nic6_on(self, ssh, sid, network_id):
        """Turns the IPv6 vNIC on

        Required because guest images usually set only the first vNIC on boot.
        Searches for the IPv6 vNIC's MAC and brings it up.

        @param ssh: RemoteClient ssh instance to server
        @param sid: server uuid
        @param network_id: the network id the NIC is connected to
        """
        ports = [
            p["mac_address"] for p in
            self.os_admin.ports_client.list_ports(
                device_id=sid, network_id=network_id)['ports']
        ]

        self.assertEqual(1, len(ports),
                         message=("Multiple IPv6 ports found on network %s. "
                                  "ports: %s")
                         % (network_id, ports))
        mac6 = ports[0]
        nic = ssh.get_nic_name_by_mac(mac6)
        # NOTE(slaweq): on RHEL based OS ifcfg file for new interface is
        # needed to make IPv6 working on it, so if
        # /etc/sysconfig/network-scripts directory exists ifcfg-%(nic)s file
        # should be added in it
        if self._sysconfig_network_scripts_dir_exists(ssh):
            try:
                ssh.exec_command(
                    'echo -e "DEVICE=%(nic)s\\nNAME=%(nic)s\\nIPV6INIT=yes" | '
                    'sudo tee /etc/sysconfig/network-scripts/ifcfg-%(nic)s; '
                    'sudo nmcli connection reload' % {'nic': nic})
                ssh.exec_command('sudo nmcli connection up %s' % nic)
            except exceptions.SSHExecCommandFailed as e:
                # NOTE(slaweq): Sometimes it can happen that this SSH command
                # will fail because of some error from network manager in
                # guest os.
                # But even then doing ip link set up below is fine and
                # IP address should be configured properly.
                LOG.debug("Error during restarting %(nic)s interface on "
                          "instance. Error message: %(error)s",
                          {'nic': nic, 'error': e})
        ssh.exec_command("sudo ip link set %s up" % nic)

    def _sysconfig_network_scripts_dir_exists(self, ssh):
        return "False" not in ssh.exec_command(
            'test -d /etc/sysconfig/network-scripts/ || echo "False"')

    def _prepare_and_test(self, address6_mode, n_subnets6=1, dualnet=False):
        net_list = self.prepare_network(address6_mode=address6_mode,
                                        n_subnets6=n_subnets6,
                                        dualnet=dualnet)

        sshv4_1, ips_from_api_1, srv1 = self.prepare_server(networks=net_list)
        sshv4_2, ips_from_api_2, srv2 = self.prepare_server(networks=net_list)

        def guest_has_address(ssh, addr):
            return addr in ssh.exec_command("ip address")

        # Turn on 2nd NIC for Cirros when dualnet
        if dualnet:
            _, network_v6 = net_list
            self.turn_nic6_on(sshv4_1, srv1['id'], network_v6['id'])
            self.turn_nic6_on(sshv4_2, srv2['id'], network_v6['id'])

        # get addresses assigned to vNIC as reported by 'ip address' utility
        ips_from_ip_1 = sshv4_1.exec_command("ip address")
        ips_from_ip_2 = sshv4_2.exec_command("ip address")
        self.assertIn(ips_from_api_1['4'], ips_from_ip_1)
        self.assertIn(ips_from_api_2['4'], ips_from_ip_2)
        for i in range(n_subnets6):
            # v6 should be configured since the image supports it
            # It can take time for ipv6 automatic address to get assigned
            for srv, ssh, ips in (
                    (srv1, sshv4_1, ips_from_api_1),
                    (srv2, sshv4_2, ips_from_api_2)):
                ip = ips['6'][i]
                result = test_utils.call_until_true(
                    guest_has_address,
                    CONF.validation.ping_timeout, 1, ssh, ip)
                if not result:
                    self.log_console_output(servers=[srv])
                    self.fail(
                        'Address %s not configured for instance %s, '
                        'ip address output is\n%s' %
                        (ip, srv['id'], ssh.exec_command("ip address")))

        self.check_remote_connectivity(sshv4_1, ips_from_api_2['4'])
        self.check_remote_connectivity(sshv4_2, ips_from_api_1['4'])

        for i in range(n_subnets6):
            self.check_remote_connectivity(sshv4_1,
                                           ips_from_api_2['6'][i])
            self.check_remote_connectivity(sshv4_1,
                                           self.subnets_v6[i]['gateway_ip'])
            self.check_remote_connectivity(sshv4_2,
                                           ips_from_api_1['6'][i])
            self.check_remote_connectivity(sshv4_2,
                                           self.subnets_v6[i]['gateway_ip'])

    @decorators.attr(type='slow')
    @decorators.idempotent_id('2c92df61-29f0-4eaa-bee3-7c65bef62a43')
    @utils.services('compute', 'network')
    def test_slaac_from_os(self):
        self._prepare_and_test(address6_mode='slaac')

    @decorators.attr(type='slow')
    @decorators.idempotent_id('d7e1f858-187c-45a6-89c9-bdafde619a9f')
    @utils.services('compute', 'network')
    def test_dhcp6_stateless_from_os(self):
        self._prepare_and_test(address6_mode='dhcpv6-stateless')

    @decorators.attr(type='slow')
    @decorators.idempotent_id('7ab23f41-833b-4a16-a7c9-5b42fe6d4123')
    @utils.services('compute', 'network')
    def test_multi_prefix_dhcpv6_stateless(self):
        self._prepare_and_test(address6_mode='dhcpv6-stateless', n_subnets6=2)

    @decorators.attr(type='slow')
    @decorators.idempotent_id('dec222b1-180c-4098-b8c5-cc1b8342d611')
    @utils.services('compute', 'network')
    def test_multi_prefix_slaac(self):
        self._prepare_and_test(address6_mode='slaac', n_subnets6=2)

    @decorators.attr(type='slow')
    @decorators.idempotent_id('b6399d76-4438-4658-bcf5-0d6c8584fde2')
    @utils.services('compute', 'network')
    def test_dualnet_slaac_from_os(self):
        self._prepare_and_test(address6_mode='slaac', dualnet=True)

    @decorators.attr(type='slow')
    @decorators.idempotent_id('76f26acd-9688-42b4-bc3e-cd134c4cb09e')
    @utils.services('compute', 'network')
    def test_dualnet_dhcp6_stateless_from_os(self):
        self._prepare_and_test(address6_mode='dhcpv6-stateless', dualnet=True)

    @decorators.attr(type='slow')
    @decorators.idempotent_id('cf1c4425-766b-45b8-be35-e2959728eb00')
    @utils.services('compute', 'network')
    def test_dualnet_multi_prefix_dhcpv6_stateless(self):
        self._prepare_and_test(address6_mode='dhcpv6-stateless', n_subnets6=2,
                               dualnet=True)

    @decorators.idempotent_id('9178ad42-10e4-47e9-8987-e02b170cc5cd')
    @decorators.attr(type='slow')
    @utils.services('compute', 'network')
    def test_dualnet_multi_prefix_slaac(self):
        self._prepare_and_test(address6_mode='slaac', n_subnets6=2,
                               dualnet=True)



# Copyright 2017 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class VersionsClient(rest_client.RestClient):
    api_version = "v3"

    def list_versions(self):
        """List API versions"""
        version_url = self._get_base_version_url()

        resp, body = self.raw_request(version_url, 'GET')
        self._error_checker(resp, body)

        self.expected_success(300, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
import tempest.test

CONF = config.CONF


class BaseIdentityTest(tempest.test.BaseTestCase):

    @classmethod
    def setup_credentials(cls):
        # Create no network resources for these test.
        cls.set_network_resources()
        super(BaseIdentityTest, cls).setup_credentials()

    @classmethod
    def disable_user(cls, user_name):
        user = cls.get_user_by_name(user_name)
        cls.users_client.update_user_enabled(user['id'], enabled=False)

    @classmethod
    def disable_tenant(cls, tenant_name):
        tenant = cls.get_tenant_by_name(tenant_name)
        cls.tenants_client.update_tenant(tenant['id'], enabled=False)

    @classmethod
    def get_user_by_name(cls, name, domain_id=None):
        if domain_id:
            params = {'domain_id': domain_id}
            users = cls.users_client.list_users(**params)['users']
        else:
            users = cls.users_client.list_users()['users']
        user = [u for u in users if u['name'] == name]
        if user:
            return user[0]

    @classmethod
    def get_tenant_by_name(cls, name):
        try:
            tenants = cls.tenants_client.list_tenants()['tenants']
        except AttributeError:
            tenants = cls.projects_client.list_projects()['projects']
        tenant = [t for t in tenants if t['name'] == name]
        if tenant:
            return tenant[0]

    @classmethod
    def get_role_by_name(cls, name):
        roles = cls.roles_client.list_roles()['roles']
        role = [r for r in roles if r['name'] == name]
        if role:
            return role[0]

    def create_test_user(self, **kwargs):
        if kwargs.get('password', None) is None:
            kwargs['password'] = data_utils.rand_password()
        if 'name' not in kwargs:
            kwargs['name'] = data_utils.rand_name('test_user')
        if 'email' not in kwargs:
            kwargs['email'] = kwargs['name'] + '@testmail.tm'

        user = self.users_client.create_user(**kwargs)['user']
        # Delete the user at the end of the test
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.users_client.delete_user, user['id'])
        return user

    def setup_test_role(self, name=None, domain_id=None):
        """Set up a test role."""
        params = {'name': name or data_utils.rand_name('test_role')}
        if domain_id:
            params['domain_id'] = domain_id

        role = self.roles_client.create_role(**params)['role']
        # Delete the role at the end of the test
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.roles_client.delete_role, role['id'])
        return role


class BaseIdentityV2Test(BaseIdentityTest):

    credentials = ['primary']

    # identity v2 tests should obtain tokens and create accounts via v2
    # regardless of the configured CONF.identity.auth_version
    identity_version = 'v2'

    @classmethod
    def setup_clients(cls):
        super(BaseIdentityV2Test, cls).setup_clients()
        cls.non_admin_client = cls.os_primary.identity_public_client
        cls.non_admin_token_client = cls.os_primary.token_client
        cls.non_admin_tenants_client = cls.os_primary.tenants_public_client
        cls.non_admin_users_client = cls.os_primary.users_public_client


class BaseIdentityV2AdminTest(BaseIdentityV2Test):

    credentials = ['primary', 'admin']

    # NOTE(andreaf) Identity tests work with credentials, so it is safer
    # for them to always use disposable credentials. Forcing dynamic creds
    # on regular identity tests would be however to restrictive, since it
    # would prevent any identity test from being executed against clouds where
    # admin credentials are not available.
    # Since All admin tests require admin credentials to be
    # executed, so this will not impact the ability to execute tests.
    force_tenant_isolation = True

    @classmethod
    def skip_checks(cls):
        super(BaseIdentityV2AdminTest, cls).skip_checks()
        if not CONF.identity_feature_enabled.api_v2_admin:
            raise cls.skipException('Identity v2 admin not available')

    @classmethod
    def setup_clients(cls):
        super(BaseIdentityV2AdminTest, cls).setup_clients()
        cls.client = cls.os_admin.identity_client
        cls.non_admin_client = cls.os_primary.identity_client
        cls.token_client = cls.os_admin.token_client
        cls.tenants_client = cls.os_admin.tenants_client
        cls.non_admin_tenants_client = cls.os_primary.tenants_client
        cls.roles_client = cls.os_admin.roles_client
        cls.non_admin_roles_client = cls.os_primary.roles_client
        cls.users_client = cls.os_admin.users_client
        cls.non_admin_users_client = cls.os_primary.users_client
        cls.services_client = cls.os_admin.identity_services_client
        cls.endpoints_client = cls.os_admin.endpoints_client

    @classmethod
    def resource_setup(cls):
        super(BaseIdentityV2AdminTest, cls).resource_setup()
        cls.projects_client = cls.tenants_client

    def setup_test_user(self, password=None):
        """Set up a test user."""
        tenant = self.setup_test_tenant()
        user = self.create_test_user(tenantId=tenant['id'], password=password)
        return user

    def setup_test_tenant(self, **kwargs):
        """Set up a test tenant."""
        if 'name' not in kwargs:
            kwargs['name'] = data_utils.rand_name('test_tenant')
        if 'description' not in kwargs:
            kwargs['description'] = data_utils.rand_name('desc')
        tenant = self.projects_client.create_tenant(**kwargs)['tenant']
        # Delete the tenant at the end of the test
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.tenants_client.delete_tenant, tenant['id'])
        return tenant


class BaseIdentityV3Test(BaseIdentityTest):

    credentials = ['primary']

    # identity v3 tests should obtain tokens and create accounts via v3
    # regardless of the configured CONF.identity.auth_version
    identity_version = 'v3'

    @classmethod
    def setup_clients(cls):
        super(BaseIdentityV3Test, cls).setup_clients()
        cls.non_admin_client = cls.os_primary.identity_v3_client
        cls.non_admin_users_client = cls.os_primary.users_v3_client
        cls.non_admin_token = cls.os_primary.token_v3_client
        cls.non_admin_projects_client = cls.os_primary.projects_client
        cls.non_admin_catalog_client = cls.os_primary.catalog_client
        cls.non_admin_versions_client =\
            cls.os_primary.identity_versions_v3_client
        cls.non_admin_app_creds_client = \
            cls.os_primary.application_credentials_client
        cls.non_admin_access_rules_client = cls.os_primary.access_rules_client


class BaseIdentityV3AdminTest(BaseIdentityV3Test):

    credentials = ['primary', 'admin']

    # NOTE(andreaf) Identity tests work with credentials, so it is safer
    # for them to always use disposable credentials. Forcing dynamic creds
    # on regular identity tests would be however to restrictive, since it
    # would prevent any identity test from being executed against clouds where
    # admin credentials are not available.
    # Since All admin tests require admin credentials to be
    # executed, so this will not impact the ability to execute tests.
    force_tenant_isolation = True

    @classmethod
    def setup_clients(cls):
        super(BaseIdentityV3AdminTest, cls).setup_clients()
        cls.client = cls.os_admin.identity_v3_client
        cls.domains_client = cls.os_admin.domains_client
        cls.users_client = cls.os_admin.users_v3_client
        cls.trusts_client = cls.os_admin.trusts_client
        cls.roles_client = cls.os_admin.roles_v3_client
        cls.inherited_roles_client = cls.os_admin.inherited_roles_client
        cls.token = cls.os_admin.token_v3_client
        cls.endpoints_client = cls.os_admin.endpoints_v3_client
        cls.regions_client = cls.os_admin.regions_client
        cls.services_client = cls.os_admin.identity_services_v3_client
        cls.policies_client = cls.os_admin.policies_client
        cls.creds_client = cls.os_admin.credentials_client
        cls.groups_client = cls.os_admin.groups_client
        cls.projects_client = cls.os_admin.projects_client
        cls.role_assignments = cls.os_admin.role_assignments_client
        cls.oauth_consumers_client = cls.os_admin.oauth_consumers_client
        cls.oauth_token_client = cls.os_admin.oauth_token_client
        cls.domain_config_client = cls.os_admin.domain_config_client
        cls.endpoint_filter_client = cls.os_admin.endpoint_filter_client
        cls.endpoint_groups_client = cls.os_admin.endpoint_groups_client
        cls.project_tags_client = cls.os_admin.project_tags_client

        if CONF.identity.admin_domain_scope:
            # NOTE(andreaf) When keystone policy requires it, the identity
            # admin clients for these tests shall use 'domain' scoped tokens.
            # As the client manager is already created by the base class,
            # we set the scope for the inner auth provider.
            cls.os_admin.auth_provider.scope = 'domain'

    @classmethod
    def disable_user(cls, user_name, domain_id=None):
        user = cls.get_user_by_name(user_name, domain_id)
        cls.users_client.update_user(user['id'], name=user_name, enabled=False)

    @classmethod
    def create_domain(cls, **kwargs):
        """Create a domain."""
        if 'name' not in kwargs:
            kwargs['name'] = data_utils.rand_name('test_domain')
        if 'description' not in kwargs:
            kwargs['description'] = data_utils.rand_name('desc')
        domain = cls.domains_client.create_domain(**kwargs)['domain']
        cls.addClassResourceCleanup(test_utils.call_and_ignore_notfound_exc,
                                    cls.delete_domain, domain['id'])
        return domain

    @classmethod
    def delete_domain(cls, domain_id):
        # NOTE(mpavlase) It is necessary to disable the domain before deleting
        # otherwise it raises Forbidden exception
        cls.domains_client.update_domain(domain_id, enabled=False)
        cls.domains_client.delete_domain(domain_id)

    def setup_test_user(self, password=None):
        """Set up a test user."""
        project = self.setup_test_project()
        user = self.create_test_user(project_id=project['id'],
                                     password=password)
        return user

    def setup_test_project(self, **kwargs):
        """Set up a test project."""
        if 'name' not in kwargs:
            kwargs['name'] = data_utils.rand_name('test_project')
        if 'description' not in kwargs:
            kwargs['description'] = data_utils.rand_name('test_description')
        project = self.projects_client.create_project(**kwargs)['project']
        # Delete the project at the end of the test
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.projects_client.delete_project, project['id'])
        return project

    def setup_test_domain(self):
        """Set up a test domain."""
        domain = self.create_domain()
        # Delete the domain at the end of the test
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.delete_domain, domain['id'])
        return domain

    def setup_test_group(self, **kwargs):
        """Set up a test group."""
        if 'name' not in kwargs:
            kwargs['name'] = data_utils.rand_name(
                self.__class__.__name__ + '_test_project')
        if 'description' not in kwargs:
            kwargs['description'] = data_utils.rand_name(
                self.__class__.__name__ + '_test_description')
        group = self.groups_client.create_group(**kwargs)['group']
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.groups_client.delete_group, group['id'])
        return group


class BaseApplicationCredentialsV3Test(BaseIdentityV3Test):

    @classmethod
    def skip_checks(cls):
        super(BaseApplicationCredentialsV3Test, cls).skip_checks()
        if not CONF.identity_feature_enabled.application_credentials:
            raise cls.skipException("Application credentials are not available"
                                    " in this environment")

    @classmethod
    def resource_setup(cls):
        super(BaseApplicationCredentialsV3Test, cls).resource_setup()
        cls.user_id = cls.os_primary.credentials.user_id
        cls.project_id = cls.os_primary.credentials.project_id

    def create_application_credential(self, name=None, **kwargs):
        name = name or data_utils.rand_name('application_credential')
        application_credential = (
            self.non_admin_app_creds_client.create_application_credential(
                self.user_id, name=name, **kwargs))['application_credential']
        self.addCleanup(
            self.non_admin_app_creds_client.delete_application_credential,
            self.user_id,
            application_credential['id'])
        return application_credential



# Copyright 2014 Hewlett-Packard Development Company, L.P.
# Copyright 2016 Rackspace Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc
import copy
import datetime
import re
from urllib import parse as urlparse

from oslo_log import log as logging

from tempest.lib import exceptions
from tempest.lib.services.identity.v2 import token_client as json_v2id
from tempest.lib.services.identity.v3 import token_client as json_v3id

ISO8601_FLOAT_SECONDS = '%Y-%m-%dT%H:%M:%S.%fZ'
ISO8601_INT_SECONDS = '%Y-%m-%dT%H:%M:%SZ'
LOG = logging.getLogger(__name__)


def replace_version(url, new_version):
    parts = urlparse.urlparse(url)
    version_path = '/%s' % new_version
    path, subs = re.subn(r'(^|/)+v\d+(?:\.\d+)?',
                         version_path,
                         parts.path,
                         count=1)
    if not subs:
        path = '%s%s' % (parts.path.rstrip('/'), version_path)
    url = urlparse.urlunparse((parts.scheme,
                               parts.netloc,
                               path,
                               parts.params,
                               parts.query,
                               parts.fragment))
    return url


def apply_url_filters(url, filters):
    if filters.get('api_version', None) is not None:
        url = replace_version(url, filters['api_version'])
    parts = urlparse.urlparse(url)
    if filters.get('skip_path', None) is not None and parts.path != '':
        url = urlparse.urlunparse((parts.scheme,
                                   parts.netloc,
                                   '/',
                                   parts.params,
                                   parts.query,
                                   parts.fragment))

    return url


class AuthProvider(object, metaclass=abc.ABCMeta):
    """Provide authentication"""

    SCOPES = set(['project'])

    def __init__(self, credentials, scope='project'):
        """Auth provider __init__

        :param credentials: credentials for authentication
        :param scope: the default scope to be used by the credential providers
                      when requesting a token. Valid values depend on the
                      AuthProvider class implementation, and are defined in
                      the set SCOPES. Default value is 'project'.
        """
        if self.check_credentials(credentials):
            self.credentials = credentials
        else:
            if isinstance(credentials, Credentials):
                password = credentials.get('password')
                message = "Credentials are: " + str(credentials)
                if password is None:
                    message += " Password is not defined."
                else:
                    message += " Password is defined."
                raise exceptions.InvalidCredentials(message)
            else:
                raise TypeError("credentials object is of type %s, which is"
                                " not a valid Credentials object type." %
                                credentials.__class__.__name__)
        self._scope = None
        self.scope = scope
        self.cache = None
        self.alt_auth_data = None
        self.alt_part = None

    def __str__(self):
        return "Creds :{creds}, cached auth data: {cache}".format(
            creds=self.credentials, cache=self.cache)

    @abc.abstractmethod
    def _decorate_request(self, filters, method, url, headers=None, body=None,
                          auth_data=None):
        """Decorate request with authentication data"""
        return

    @abc.abstractmethod
    def _get_auth(self):
        return

    @abc.abstractmethod
    def _fill_credentials(self, auth_data_body):
        return

    def fill_credentials(self):
        """Fill credentials object with data from auth"""
        auth_data = self.get_auth()
        self._fill_credentials(auth_data[1])
        return self.credentials

    @classmethod
    def check_credentials(cls, credentials):
        """Verify credentials are valid."""
        return isinstance(credentials, Credentials) and credentials.is_valid()

    @property
    def auth_data(self):
        """Auth data for set scope"""
        return self.get_auth()

    @property
    def scope(self):
        """Scope used in auth requests"""
        return self._scope

    @auth_data.deleter
    def auth_data(self):
        self.clear_auth()

    def get_auth(self):
        """Returns auth from cache if available, else auth first"""
        if self.cache is None or self.is_expired(self.cache):
            self.set_auth()
        return self.cache

    def set_auth(self):
        """Forces setting auth.

        Forces setting auth, ignores cache if it exists.
        Refills credentials.
        """
        self.cache = self._get_auth()
        self._fill_credentials(self.cache[1])

    def clear_auth(self):
        """Clear access cache

        Can be called to clear the access cache so that next request
        will fetch a new token and base_url.
        """
        self.cache = None
        self.credentials.reset()

    @abc.abstractmethod
    def is_expired(self, auth_data):
        return

    def auth_request(self, method, url, headers=None, body=None, filters=None):
        """Obtains auth data and decorates a request with that.

        :param method: HTTP method of the request
        :param url: relative URL of the request (path)
        :param headers: HTTP headers of the request
        :param body: HTTP body in case of POST / PUT
        :param filters: select a base URL out of the catalog
        :return: a Tuple (url, headers, body)
        """
        orig_req = dict(url=url, headers=headers, body=body)

        auth_url, auth_headers, auth_body = self._decorate_request(
            filters, method, url, headers, body)
        auth_req = dict(url=auth_url, headers=auth_headers, body=auth_body)

        # Overwrite part if the request if it has been requested
        if self.alt_part is not None:
            if self.alt_auth_data is not None:
                alt_url, alt_headers, alt_body = self._decorate_request(
                    filters, method, url, headers, body,
                    auth_data=self.alt_auth_data)
                alt_auth_req = dict(url=alt_url, headers=alt_headers,
                                    body=alt_body)
                if auth_req[self.alt_part] == alt_auth_req[self.alt_part]:
                    raise exceptions.BadAltAuth(part=self.alt_part)
                auth_req[self.alt_part] = alt_auth_req[self.alt_part]

            else:
                # If the requested part is not affected by auth, we are
                # not altering auth as expected, raise an exception
                if auth_req[self.alt_part] == orig_req[self.alt_part]:
                    raise exceptions.BadAltAuth(part=self.alt_part)
                # If alt auth data is None, skip auth in the requested part
                auth_req[self.alt_part] = orig_req[self.alt_part]

            # Next auth request will be normal, unless otherwise requested
            self.reset_alt_auth_data()

        return auth_req['url'], auth_req['headers'], auth_req['body']

    def reset_alt_auth_data(self):
        """Configure auth provider to provide valid authentication data"""
        self.alt_part = None
        self.alt_auth_data = None

    def set_alt_auth_data(self, request_part, auth_data):
        """Alternate auth data on next request

        Configure auth provider to provide alt authentication data
        on a part of the *next* auth_request. If credentials are None,
        set invalid data.

        :param request_part: request part to contain invalid auth: url,
                             headers, body
        :param auth_data: alternative auth_data from which to get the
                          invalid data to be injected
        """
        self.alt_part = request_part
        self.alt_auth_data = auth_data

    @abc.abstractmethod
    def base_url(self, filters, auth_data=None):
        """Extracts the base_url based on provided filters"""
        return

    @scope.setter
    def scope(self, value):
        """Set the scope to be used in token requests

        :param scope: scope to be used. If the scope is different, clear caches
        """
        if value not in self.SCOPES:
            raise exceptions.InvalidScope(
                scope=value, auth_provider=self.__class__.__name__)
        if value != self.scope:
            self.clear_auth()
            self._scope = value


class KeystoneAuthProvider(AuthProvider):

    EXPIRY_DATE_FORMATS = (ISO8601_FLOAT_SECONDS, ISO8601_INT_SECONDS)

    token_expiry_threshold = datetime.timedelta(seconds=60)

    def __init__(self, credentials, auth_url,
                 disable_ssl_certificate_validation=None,
                 ca_certs=None, trace_requests=None, scope='project',
                 http_timeout=None, proxy_url=None):
        super(KeystoneAuthProvider, self).__init__(credentials, scope)
        self.dscv = disable_ssl_certificate_validation
        self.ca_certs = ca_certs
        self.trace_requests = trace_requests
        self.http_timeout = http_timeout
        self.proxy_url = proxy_url
        self.auth_url = auth_url
        self.auth_client = self._auth_client(auth_url)

    def _decorate_request(self, filters, method, url, headers=None, body=None,
                          auth_data=None):
        if auth_data is None:
            auth_data = self.get_auth()
        token, _ = auth_data
        base_url = self.base_url(filters=filters, auth_data=auth_data)
        # build authenticated request
        # returns new request, it does not touch the original values
        _headers = copy.deepcopy(headers) if headers is not None else {}
        _headers['X-Auth-Token'] = str(token)
        if url is None or url == "":
            _url = base_url
        else:
            # Join base URL and url, and remove multiple contiguous slashes
            _url = "/".join([base_url, url])
            parts = [x for x in urlparse.urlparse(_url)]
            parts[2] = re.sub("/{2,}", "/", parts[2])
            _url = urlparse.urlunparse(parts)
        # no change to method or body
        return str(_url), _headers, body

    @abc.abstractmethod
    def _auth_client(self):
        return

    @abc.abstractmethod
    def _auth_params(self):
        """Auth parameters to be passed to the token request

        By default all fields available in Credentials are passed to the
        token request. Scope may affect this.
        """
        return

    def _get_auth(self):
        # Bypasses the cache
        auth_func = getattr(self.auth_client, 'get_token')
        auth_params = self._auth_params()

        # returns token, auth_data
        token, auth_data = auth_func(**auth_params)
        return token, auth_data

    def _parse_expiry_time(self, expiry_string):
        expiry = None
        for date_format in self.EXPIRY_DATE_FORMATS:
            try:
                expiry = datetime.datetime.strptime(
                    expiry_string, date_format)
            except ValueError:
                pass
        if expiry is None:
            raise ValueError(
                "time data '{data}' does not match any of the "
                "expected formats: {formats}".format(
                    data=expiry_string, formats=self.EXPIRY_DATE_FORMATS))
        return expiry

    def get_token(self):
        return self.get_auth()[0]


class KeystoneV2AuthProvider(KeystoneAuthProvider):
    """Provides authentication based on the Identity V2 API

    The Keystone Identity V2 API defines both unscoped and project scoped
    tokens. This auth provider only implements 'project'.
    """

    SCOPES = set(['project'])

    def _auth_client(self, auth_url):
        return json_v2id.TokenClient(
            auth_url, disable_ssl_certificate_validation=self.dscv,
            ca_certs=self.ca_certs, trace_requests=self.trace_requests,
            http_timeout=self.http_timeout, proxy_url=self.proxy_url)

    def _auth_params(self):
        """Auth parameters to be passed to the token request

        All fields available in Credentials are passed to the token request.
        """
        return dict(
            user=self.credentials.username,
            password=self.credentials.password,
            tenant=self.credentials.tenant_name,
            auth_data=True)

    def _fill_credentials(self, auth_data_body):
        tenant = auth_data_body['token']['tenant']
        user = auth_data_body['user']
        if self.credentials.tenant_name is None:
            self.credentials.tenant_name = tenant['name']
        if self.credentials.tenant_id is None:
            self.credentials.tenant_id = tenant['id']
        if self.credentials.username is None:
            self.credentials.username = user['name']
        if self.credentials.user_id is None:
            self.credentials.user_id = user['id']

    def base_url(self, filters, auth_data=None):
        """Base URL from catalog

        :param filters: Used to filter results

        Filters can be:

        - service: service type name such as compute, image, etc.
        - region: service region name
        - name: service name, only if service exists
        - endpoint_type: type of endpoint such as
            adminURL, publicURL, internalURL
        - api_version: the version of api used to replace catalog version
        - skip_path: skips the suffix path of the url and uses base URL

        :rtype: string
        :return: url with filters applied
        """
        if auth_data is None:
            auth_data = self.get_auth()
        _, _auth_data = auth_data
        service = filters.get('service')
        region = filters.get('region')
        name = filters.get('name')
        endpoint_type = filters.get('endpoint_type', 'publicURL')

        if service is None:
            raise exceptions.EndpointNotFound("No service provided")

        _base_url = None
        for ep in _auth_data['serviceCatalog']:
            if ep["type"] == service:
                if name is not None and ep["name"] != name:
                    continue
                for _ep in ep['endpoints']:
                    if region is not None and _ep['region'] == region:
                        _base_url = _ep.get(endpoint_type)
                if not _base_url:
                    # No region or name matching, use the first
                    _base_url = ep['endpoints'][0].get(endpoint_type)
                break
        if _base_url is None:
            raise exceptions.EndpointNotFound(
                "service: %s, region: %s, endpoint_type: %s, name: %s" %
                (service, region, endpoint_type, name))
        return apply_url_filters(_base_url, filters)

    def is_expired(self, auth_data):
        _, access = auth_data
        expiry = self._parse_expiry_time(access['token']['expires'])
        return (expiry - self.token_expiry_threshold <=
                datetime.datetime.utcnow())


class KeystoneV3AuthProvider(KeystoneAuthProvider):
    """Provides authentication based on the Identity V3 API"""

    SCOPES = set(['system', 'project', 'domain', 'unscoped', None])

    def _auth_client(self, auth_url):
        return json_v3id.V3TokenClient(
            auth_url, disable_ssl_certificate_validation=self.dscv,
            ca_certs=self.ca_certs, trace_requests=self.trace_requests,
            http_timeout=self.http_timeout, proxy_url=self.proxy_url)

    def _auth_params(self):
        """Auth parameters to be passed to the token request

        Fields available in Credentials are passed to the token request,
        depending on the value of scope. Valid values for scope are: "project",
        "domain", or "system". Any other string (e.g. "unscoped") or None will
        lead to an unscoped token request.
        """

        auth_params = dict(
            user_id=self.credentials.user_id,
            username=self.credentials.username,
            user_domain_id=self.credentials.user_domain_id,
            user_domain_name=self.credentials.user_domain_name,
            password=self.credentials.password,
            auth_data=True)

        if self.scope == 'project':
            auth_params.update(
                project_domain_id=self.credentials.project_domain_id,
                project_domain_name=self.credentials.project_domain_name,
                project_id=self.credentials.project_id,
                project_name=self.credentials.project_name)

        if self.scope == 'domain':
            auth_params.update(
                domain_id=self.credentials.domain_id,
                domain_name=self.credentials.domain_name)

        if self.scope == 'system':
            auth_params.update(system='all')

        return auth_params

    def _fill_credentials(self, auth_data_body):
        # project, domain, or system depending on the scope
        project = auth_data_body.get('project', None)
        domain = auth_data_body.get('domain', None)
        system = auth_data_body.get('system', None)
        # user is always there
        user = auth_data_body['user']
        # Set project fields
        if project is not None:
            if self.credentials.project_name is None:
                self.credentials.project_name = project['name']
            if self.credentials.project_id is None:
                self.credentials.project_id = project['id']
            if self.credentials.project_domain_id is None:
                self.credentials.project_domain_id = project['domain']['id']
            if self.credentials.project_domain_name is None:
                self.credentials.project_domain_name = (
                    project['domain']['name'])
        # Set domain fields
        if domain is not None:
            if self.credentials.domain_id is None:
                self.credentials.domain_id = domain['id']
            if self.credentials.domain_name is None:
                self.credentials.domain_name = domain['name']
        # Set system scope
        if system is not None:
            self.credentials.system = 'all'
        # Set user fields
        if self.credentials.username is None:
            self.credentials.username = user['name']
        if self.credentials.user_id is None:
            self.credentials.user_id = user['id']
        if self.credentials.user_domain_id is None:
            self.credentials.user_domain_id = user['domain']['id']
        if self.credentials.user_domain_name is None:
            self.credentials.user_domain_name = user['domain']['name']

    def base_url(self, filters, auth_data=None):
        """Base URL from catalog

        If scope is not 'project', it may be that there is not catalog in
        the auth_data. In such case, as long as the requested service is
        'identity', we can use the original auth URL to build the base_url.

        :param filters: Used to filter results

        Filters can be:

        - service: service type name such as compute, image, etc.
        - region: service region name
        - name: service name, only if service exists
        - endpoint_type: type of endpoint such as
            adminURL, publicURL, internalURL
        - api_version: the version of api used to replace catalog version
        - skip_path: skips the suffix path of the url and uses base URL

        :rtype: string
        :return: url with filters applied
        """
        if auth_data is None:
            auth_data = self.get_auth()
        _, _auth_data = auth_data
        service = filters.get('service')
        region = filters.get('region')
        name = filters.get('name')
        endpoint_type = filters.get('endpoint_type', 'public')

        if service is None:
            raise exceptions.EndpointNotFound("No service provided")

        if 'URL' in endpoint_type:
            endpoint_type = endpoint_type.replace('URL', '')
        _base_url = None
        catalog = _auth_data.get('catalog', [])

        # Select entries with matching service type
        service_catalog = [ep for ep in catalog if ep['type'] == service]
        if service_catalog:
            if name is not None:
                service_catalog = (
                    [ep for ep in service_catalog if ep['name'] == name])
                if service_catalog:
                    service_catalog = service_catalog[0]['endpoints']
                else:
                    raise exceptions.EndpointNotFound(name)
            else:
                service_catalog = service_catalog[0]['endpoints']
        else:
            if not catalog and service == 'identity':
                # NOTE(andreaf) If there's no catalog at all and the service
                # is identity, it's a valid use case. Having a non-empty
                # catalog with no identity in it is not valid instead.
                msg = ('Got an empty catalog. Scope: %s. '
                       'Falling back to configured URL for %s: %s')
                LOG.debug(msg, self.scope, service, self.auth_url)
                return apply_url_filters(self.auth_url, filters)
            else:
                # No matching service
                msg = ('No matching service found in the catalog.\n'
                       'Scope: %s, Credentials: %s\n'
                       'Auth data: %s\n'
                       'Service: %s, Region: %s, endpoint_type: %s\n'
                       'Catalog: %s')
                raise exceptions.EndpointNotFound(msg % (
                    self.scope, self.credentials, _auth_data, service, region,
                    endpoint_type, catalog))
        # Filter by endpoint type (interface)
        filtered_catalog = [ep for ep in service_catalog if
                            ep['interface'] == endpoint_type]
        if not filtered_catalog:
            # No matching type, keep all and try matching by region at least
            filtered_catalog = service_catalog
        # Filter by region
        filtered_catalog = [ep for ep in filtered_catalog if
                            ep['region'] == region]
        if not filtered_catalog:
            # No matching region (or name), take the first endpoint
            filtered_catalog = [service_catalog[0]]
        # There should be only one match. If not take the first.
        _base_url = filtered_catalog[0].get('url', None)
        if _base_url is None:
            raise exceptions.EndpointNotFound(service)
        return apply_url_filters(_base_url, filters)

    def is_expired(self, auth_data):
        _, access = auth_data
        expiry = self._parse_expiry_time(access['expires_at'])
        return (expiry - self.token_expiry_threshold <=
                datetime.datetime.utcnow())


def is_identity_version_supported(identity_version):
    return identity_version in IDENTITY_VERSION


def get_credentials(auth_url, fill_in=True, identity_version='v2',
                    disable_ssl_certificate_validation=None, ca_certs=None,
                    trace_requests=None, http_timeout=None, proxy_url=None,
                    **kwargs):
    """Builds a credentials object based on the configured auth_version

    :param auth_url (string): Full URI of the OpenStack Identity API(Keystone)
           which is used to fetch the token from Identity service.
    :param fill_in (boolean): obtain a token and fill in all credential
           details provided by the identity service. When fill_in is not
           specified, credentials are not validated. Validation can be invoked
           by invoking ``is_valid()``
    :param identity_version (string): identity API version is used to
           select the matching auth provider and credentials class
    :param disable_ssl_certificate_validation: whether to enforce SSL
           certificate validation in SSL API requests to the auth system
    :param ca_certs: CA certificate bundle for validation of certificates
           in SSL API requests to the auth system
    :param trace_requests: trace in log API requests to the auth system
    :param http_timeout: timeout in seconds to wait for the http request to
           return
    :param proxy_url: URL of HTTP(s) proxy used when fill_in is True
    :param kwargs (dict): Dict of credential key/value pairs

    Examples:

        Returns credentials from the provided parameters:
        >>> get_credentials(username='foo', password='bar')

        Returns credentials including IDs:
        >>> get_credentials(username='foo', password='bar', fill_in=True)
    """
    if not is_identity_version_supported(identity_version):
        raise exceptions.InvalidIdentityVersion(
            identity_version=identity_version)

    credential_class, auth_provider_class = IDENTITY_VERSION.get(
        identity_version)

    creds = credential_class(**kwargs)
    # Fill in the credentials fields that were not specified
    if fill_in:
        dscv = disable_ssl_certificate_validation
        auth_provider = auth_provider_class(
            creds, auth_url, disable_ssl_certificate_validation=dscv,
            ca_certs=ca_certs, trace_requests=trace_requests,
            http_timeout=http_timeout, proxy_url=proxy_url)
        creds = auth_provider.fill_credentials()
    return creds


class Credentials(object):
    """Set of credentials for accessing OpenStack services

    ATTRIBUTES: list of valid class attributes representing credentials.
    """

    ATTRIBUTES = []
    COLLISIONS = []

    def __init__(self, **kwargs):
        """Enforce the available attributes at init time (only).

        Additional attributes can still be set afterwards if tests need
        to do so.
        """
        self._initial = kwargs
        self._apply_credentials(kwargs)

    def _apply_credentials(self, attr):
        for (key1, key2) in self.COLLISIONS:
            val1 = attr.get(key1)
            val2 = attr.get(key2)
            if val1 and val2 and val1 != val2:
                msg = ('Cannot have conflicting values for %s and %s' %
                       (key1, key2))
                raise exceptions.InvalidCredentials(msg)
        for key in attr:
            if key in self.ATTRIBUTES:
                if attr[key] is not None:
                    setattr(self, key, attr[key])
            else:
                msg = '%s is not a valid attr for %s' % (key, self.__class__)
                raise exceptions.InvalidCredentials(msg)

    def __str__(self):
        """Represent only attributes included in self.ATTRIBUTES"""
        attrs = [attr for attr in self.ATTRIBUTES if attr != 'password']
        _repr = dict((k, getattr(self, k)) for k in attrs)
        return str(_repr)

    def __eq__(self, other):
        """Credentials are equal if attributes in self.ATTRIBUTES are equal"""
        return str(self) == str(other)

    def __ne__(self, other):
        """Contrary to the __eq__"""
        return not self.__eq__(other)

    def __getattr__(self, key):
        # If an attribute is set, __getattr__ is not invoked
        # If an attribute is not set, and it is a known one, return None
        if key in self.ATTRIBUTES:
            return None
        else:
            raise AttributeError

    def __delitem__(self, key):
        # For backwards compatibility, support dict behaviour
        if key in self.ATTRIBUTES:
            delattr(self, key)
        else:
            raise AttributeError

    def get(self, item, default=None):
        # In this patch act as dict for backward compatibility
        try:
            return getattr(self, item)
        except AttributeError:
            return default

    def get_init_attributes(self):
        return self._initial.keys()

    def is_valid(self):
        raise NotImplementedError

    def reset(self):
        # First delete all known attributes
        for key in self.ATTRIBUTES:
            if getattr(self, key) is not None:
                delattr(self, key)
        # Then re-apply initial setup
        self._apply_credentials(self._initial)


class KeystoneV2Credentials(Credentials):

    ATTRIBUTES = ['username', 'password', 'tenant_name', 'user_id',
                  'tenant_id', 'project_id', 'project_name']
    COLLISIONS = [('project_name', 'tenant_name'), ('project_id', 'tenant_id')]

    def __str__(self):
        """Represent only attributes included in self.ATTRIBUTES"""
        attrs = [attr for attr in self.ATTRIBUTES if attr != 'password']
        _repr = dict((k, getattr(self, k)) for k in attrs)
        return str(_repr)

    def __setattr__(self, key, value):
        # NOTE(andreaf) In order to ease the migration towards 'project' we
        # support v2 credentials configured with 'project' and translate it
        # to tenant on the fly. The original kwargs are stored for clients
        # that may rely on them. We also set project when tenant is defined
        # so clients can rely on project being part of credentials.
        parent = super(KeystoneV2Credentials, self)
        # for project_* set tenant only
        if key == 'project_id':
            parent.__setattr__('tenant_id', value)
        elif key == 'project_name':
            parent.__setattr__('tenant_name', value)
        if key == 'tenant_id':
            parent.__setattr__('project_id', value)
        elif key == 'tenant_name':
            parent.__setattr__('project_name', value)
        # trigger default behaviour for all attributes
        parent.__setattr__(key, value)

    def is_valid(self):
        """Check of credentials (no API call)

        Minimum set of valid credentials, are username and password.
        Tenant is optional.
        """
        return None not in (self.username, self.password)


class KeystoneV3Credentials(Credentials):
    """Credentials suitable for the Keystone Identity V3 API"""

    ATTRIBUTES = ['domain_id', 'domain_name', 'password', 'username',
                  'project_domain_id', 'project_domain_name', 'project_id',
                  'project_name', 'tenant_id', 'tenant_name', 'user_domain_id',
                  'user_domain_name', 'user_id', 'system']
    COLLISIONS = [('project_name', 'tenant_name'), ('project_id', 'tenant_id')]

    def __setattr__(self, key, value):
        parent = super(KeystoneV3Credentials, self)
        # for tenant_* set both project and tenant
        if key == 'tenant_id':
            parent.__setattr__('project_id', value)
        elif key == 'tenant_name':
            parent.__setattr__('project_name', value)
        # for project_* set both project and tenant
        if key == 'project_id':
            parent.__setattr__('tenant_id', value)
        elif key == 'project_name':
            parent.__setattr__('tenant_name', value)
        # for *_domain_* set both user and project if not set yet
        if key == 'user_domain_id':
            if self.project_domain_id is None:
                parent.__setattr__('project_domain_id', value)
        if key == 'project_domain_id':
            if self.user_domain_id is None:
                parent.__setattr__('user_domain_id', value)
        if key == 'user_domain_name':
            if self.project_domain_name is None:
                parent.__setattr__('project_domain_name', value)
        if key == 'project_domain_name':
            if self.user_domain_name is None:
                parent.__setattr__('user_domain_name', value)
        # support domain_name coming from config
        if key == 'domain_name':
            if self.user_domain_name is None:
                parent.__setattr__('user_domain_name', value)
            if self.project_domain_name is None:
                parent.__setattr__('project_domain_name', value)
        # finally trigger default behaviour for all attributes
        parent.__setattr__(key, value)

    def is_valid(self):
        """Check of credentials (no API call)

        Valid combinations of v3 credentials (excluding token)
        - User id, password (optional domain)
        - User name, password and its domain id/name
        For the scope, valid combinations are:
        - None
        - Project id (optional domain)
        - Project name and its domain id/name
        - Domain id
        - Domain name
        """
        valid_user_domain = any(
            [self.user_domain_id is not None,
             self.user_domain_name is not None])
        valid_project_domain = any(
            [self.project_domain_id is not None,
             self.project_domain_name is not None])
        valid_user = any(
            [self.user_id is not None,
             self.username is not None and valid_user_domain])
        valid_project_scope = any(
            [self.project_name is None and self.project_id is None,
             self.project_id is not None,
             self.project_name is not None and valid_project_domain])
        valid_domain_scope = any(
            [self.domain_id is None and self.domain_name is None,
             self.domain_id or self.domain_name])
        return all([self.password is not None,
                    valid_user,
                    valid_project_scope and valid_domain_scope])


IDENTITY_VERSION = {'v2': (KeystoneV2Credentials, KeystoneV2AuthProvider),
                    'v3': (KeystoneV3Credentials, KeystoneV3AuthProvider)}



# Copyright 2013 Huawei Technologies Co.,LTD
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from testtools import matchers

from tempest.api.volume import base
from tempest.lib import decorators


class VolumesMetadataTest(base.BaseVolumeTest):
    """Test volume metadata"""

    @classmethod
    def resource_setup(cls):
        super(VolumesMetadataTest, cls).resource_setup()
        # Create a volume
        cls.volume = cls.create_volume()

    def tearDown(self):
        # Update the metadata to {}
        self.volumes_client.update_volume_metadata(self.volume['id'], {})
        super(VolumesMetadataTest, self).tearDown()

    @decorators.idempotent_id('6f5b125b-f664-44bf-910f-751591fe5769')
    def test_crud_volume_metadata(self):
        """Test creating, getting, updating and deleting of volume metadata"""
        # Create metadata for the volume
        metadata = {"key1": "value1",
                    "key2": "value2",
                    "key3": "value3",
                    "key4": "<value&special_chars>"}
        update = {"key4": "value4",
                  "key1": "value1_update"}
        expected = {"key4": "value4"}

        body = self.volumes_client.create_volume_metadata(self.volume['id'],
                                                          metadata)['metadata']
        self.assertThat(body.items(), matchers.ContainsAll(metadata.items()))
        # Get the metadata of the volume
        body = self.volumes_client.show_volume_metadata(
            self.volume['id'])['metadata']
        self.assertThat(body.items(), matchers.ContainsAll(metadata.items()),
                        'Create metadata for the volume failed')

        # Update metadata
        body = self.volumes_client.update_volume_metadata(
            self.volume['id'], update)['metadata']
        self.assertEqual(update, body)
        body = self.volumes_client.show_volume_metadata(
            self.volume['id'])['metadata']
        self.assertEqual(update, body, 'Update metadata failed')

        # Delete one item metadata of the volume
        self.volumes_client.delete_volume_metadata_item(
            self.volume['id'], "key1")
        body = self.volumes_client.show_volume_metadata(
            self.volume['id'])['metadata']
        self.assertNotIn("key1", body)
        self.assertThat(body.items(), matchers.ContainsAll(expected.items()),
                        'Delete one item metadata of the volume failed')

    @decorators.idempotent_id('862261c5-8df4-475a-8c21-946e50e36a20')
    def test_update_show_volume_metadata_item(self):
        """Test updating and getting single volume metadata item"""
        # Update metadata item for the volume
        metadata = {"key1": "value1",
                    "key2": "value2",
                    "key3": "value3"}
        update_item = {"key3": "value3_update"}
        expect = {"key1": "value1",
                  "key2": "value2",
                  "key3": "value3_update"}
        # Create metadata for the volume
        body = self.volumes_client.create_volume_metadata(
            self.volume['id'], metadata)['metadata']
        self.assertThat(body.items(),
                        matchers.ContainsAll(metadata.items()))
        # Update metadata item
        body = self.volumes_client.update_volume_metadata_item(
            self.volume['id'], "key3", update_item)['meta']
        self.assertEqual(update_item, body)

        # Get a specific metadata item of the volume
        body = self.volumes_client.show_volume_metadata_item(
            self.volume['id'], "key3")['meta']
        self.assertEqual({"key3": expect['key3']}, body)

        # Get the metadata of the volume
        body = self.volumes_client.show_volume_metadata(
            self.volume['id'])['metadata']
        self.assertThat(body.items(), matchers.ContainsAll(expect.items()))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.network import base


class SegmentsClient(base.BaseNetworkClient):

    def create_segment(self, **kwargs):
        """Creates a segment.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#create-segment
        """
        uri = '/segments'
        post_data = {'segment': kwargs}
        return self.create_resource(uri, post_data)

    def update_segment(self, segment_id, **kwargs):
        """Updates a segment.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#update-segment
        """
        uri = '/segments/%s' % segment_id
        post_data = {'segment': kwargs}
        return self.update_resource(uri, post_data)

    def show_segment(self, segment_id, **fields):
        """Shows details of a segment.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#show-segment-details
        """
        uri = '/segments/%s' % segment_id
        return self.show_resource(uri, **fields)

    def delete_segment(self, segment_id):
        """Deletes a segment"""
        uri = '/segments/%s' % segment_id
        return self.delete_resource(uri)

    def list_segments(self, **filters):
        """Lists segments.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#list-segments
        """
        uri = '/segments'
        return self.list_resources(uri, **filters)
